from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from Testing.ZopeTestCase.utils import setupCoreSessions
from plone.testing import z2
from zope.configuration import xmlconfig
import ftw.participation.tests.builders
import transaction


TEST_USER_ID_2 = '_test_user_2_'
TEST_USER_PW_2 = 'secret'


class FtwParticipationLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'Products.PloneHotfix20160830')

        # Invoke SESSION
        setupCoreSessions(app)
        transaction.commit()

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.participation:default')
        applyProfile(portal, 'plone.formwidget.autocomplete:default')
        applyProfile(portal, 'ftw.tabbedview:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

        portal.portal_registration.addMember(TEST_USER_ID_2, TEST_USER_PW_2)


FTW_PARTICIPATION_FIXTURE = FtwParticipationLayer()
FTW_PARTICIPATION_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_PARTICIPATION_FIXTURE,), name="FtwParticipation:Integration")

FTW_PARTICIPATION_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_PARTICIPATION_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwParticipation:Functional")



from Products.Five.browser import BrowserView
from ftw.participation.interfaces import IParticipationSupport


class ParticipationActive(BrowserView):
    """Checks if the participation is active on this context.
    """

    def __call__(self):
        return IParticipationSupport.providedBy(self.context)



from AccessControl import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_parent
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from ftw.participation import _
from ftw.participation.browser.invite import InviteForm
from ftw.participation.events import InvitationRetractedEvent
from ftw.participation.events import LocalRoleRemoved
from ftw.participation.interfaces import IInvitationStorage
from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.interfaces import IParticipationSupport
from plone.app.workflow.interfaces import ISharingPageRole
from plone.registry.interfaces import IRegistry
from zExceptions import Forbidden
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.event import notify
from zope.i18n import translate
import pkg_resources

try:
    pkg_resources.get_distribution('ftw.lawgiver')
except pkg_resources.DistributionNotFound:
    HAS_LAWGIVER = False
else:
    HAS_LAWGIVER = True
    from ftw.lawgiver.interfaces import IDynamicRoleAdapter


ROLES_WHITE_LIST = ['Owner']


def get_friendly_role_names(names, context, request):
    friendly_names = []

    for name in names:
        title = None
        if HAS_LAWGIVER:
            adapter = queryMultiAdapter((context, request),
                                        IDynamicRoleAdapter,
                                        name=name)
            if adapter:
                title = adapter.get_title()

        if title is None:
            utility = queryUtility(ISharingPageRole, name=name)
            if utility:
                title = utility.title

        if title is None and name not in ROLES_WHITE_LIST:
            continue
        elif title is None:
            friendly_names.append(name)
        else:
            friendly_names.append(translate(title, context=request))
    friendly_names.sort()
    return friendly_names


def inherited(context):
    """Return True if local roles are inherited here.
    """
    return not bool(
        getattr(aq_base(context), '__ac_local_roles_block__', None))


class ManageParticipants(BrowserView):
    """Manage Participants
    """

    template = ViewPageTemplateFile('participants.pt')
    form = ViewPageTemplateFile('participants_form.pt')
    storage = None

    def __call__(self):
        form = self.request.form

        del_userids = form.get('userids', [])
        del_invitations = form.get('invitations', [])

        if form.get('form.resend-invite') and del_invitations:
            self.storage = IInvitationStorage(self.context)
            self.resend_invitations(del_invitations)


        if form.get('form.delete') and (del_userids or del_invitations):
            self.remove_users(del_userids)
            self.remove_invitations(del_invitations)

            IStatusMessage(self.request).addStatusMessage(_(u"Changes saved."),
                                                          type='info')

        elif form.get('form.cancel'):
            return self.request.RESPONSE.redirect(self.cancel_url())

        return self.template()

    def resend_invitations(self, invitation_iids):
        for iid in invitation_iids:
            self.storage.get_invitation_by_iid(iid)
            all_invitations = self.storage._invitations_list
            invite_form = InviteForm(self.context, self.request)

            for email, user_invitations in all_invitations.items():
                for invite in user_invitations:
                    if invite.iid == iid:
                        note = translate(_(u'reminder_invitation_note',
                                           default=u'This is a reminder.'),
                                         context=self.request)
                        invite_form.send_invitation(invite, email, self.get_inviter(), note)

    def get_inviter(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.getAuthenticatedMember()

    def render_form(self):
        return self.form()

    def can_manage(self):
        sm = getSecurityManager()
        has_permission = sm.checkPermission('Sharing page: Delegate roles',
                                            self.context)
        return has_permission and self.has_participation_support()

    def require_manage(self):
        if not self.can_manage():
            raise Forbidden

    def remove_invitations(self, iids):
        self.require_manage()

        storage = IInvitationStorage(self.context)

        mtool = getToolByName(self.context, 'portal_membership')
        member = mtool.getAuthenticatedMember()

        for iid in iids:
            invitation = storage.get_invitation_by_iid(iid)
            if invitation is None:
                continue

            if invitation.inviter != member.getId():
                raise Forbidden

            storage.remove_invitation(invitation)
            notify(InvitationRetractedEvent(invitation.get_target(),
                                            invitation))

    def remove_users(self, userids):
        self.require_manage()

        deletable = [p['userid'] for p in filter(
            lambda p: not p['readonly'],
            self.get_participants())]

        # we should not remove readonly participants (like myself)
        for userid in userids:
            if userid not in deletable:
                raise Forbidden
            notify(LocalRoleRemoved(self.context, userid))

        # now we need to remove the local roles recursively
        query = dict(path='/'.join(self.context.getPhysicalPath()))
        for brain in self.context.portal_catalog(query):
            obj = brain.getObject()
            obj_local_roles = dict(obj.get_local_roles())

            # do we have to change something?
            obj_to_delete = tuple(
                set(userids) & set(obj_local_roles.keys()))
            if len(obj_to_delete) > 0:
                obj.manage_delLocalRoles(userids=obj_to_delete)

        # we need to reindex the object security
        self.context.reindexObjectSecurity()

    def cancel_url(self):
        return self.context.absolute_url()

    def get_roles_settings(self):
        context = self.context
        portal = getToolByName(context, 'portal_url').getPortalObject()
        result = {}

        def get_roles(context, acquired=False):

            for userid, roles in context.get_local_roles():
                roles = list(roles)
                if 'Owner' in roles and acquired:
                    roles.remove('Owner')

                if userid not in result:
                    result[userid] = dict((role, acquired) for role in roles)
                else:
                    current_roles = result[userid]
                    for role in roles:
                        if role not in current_roles:
                            current_roles[role] = acquired

            parent = aq_parent(context)
            if parent != portal and inherited(context):
                get_roles(parent, acquired=True)

        get_roles(context)
        result = list(result.items())
        return result

    def get_participants(self):
        """Returns some items for the template. Participants are local_roles..
        """
        mtool = getToolByName(self.context, 'portal_membership')
        users = []

        for userid, roles in self.get_roles_settings():

            member = mtool.getMemberById(userid)
            # skip groups
            if member is not None:
                email = member.getProperty('email', '')
                name = member.getProperty('fullname', '')

                all_roles = roles.keys()
                inherited_roles = [r for r, v in roles.items()
                                   if v]

                item = dict(
                    userid=userid,
                    roles=get_friendly_role_names(
                        all_roles, self.context, self.request),
                    inherited_roles=get_friendly_role_names(
                        inherited_roles, self.context, self.request),
                    readonly=self.cannot_remove_user(userid),
                    type_='userids')
                if name and email:
                    item['name'] = u'%s (%s)' % (name.decode('utf-8'),
                                                 email.decode('utf-8'))
                elif name:
                    item['name'] = name.decode('utf-8')
                else:
                    item['name'] = userid.decode('utf-8')

                if item.get('roles') or item.get('inherited_roles'):
                    users.append(item)

        users.sort(key=lambda item: item['name'].lower())
        return users

    def cannot_remove_user(self, userid):
        if not self.can_manage():
            return True

        mtool = getToolByName(self.context, 'portal_membership')
        user = mtool.getMemberById(userid)

        if not user:
            return False

        elif 'Owner' in user.getRolesInContext(self.context):
            return True

        else:
            authenticated_member = mtool.getAuthenticatedMember()
            return userid == authenticated_member.getId()

    def get_pending_invitations(self):
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        mtool = getToolByName(self.context, 'portal_membership')
        storage = IInvitationStorage(portal)

        mtool = getToolByName(self.context, 'portal_membership')
        member = mtool.getAuthenticatedMember()

        invitations = []
        for invitation in storage.get_invitations_for_context(self.context):
            inviter = mtool.getMemberById(invitation.inviter)
            if inviter:
                inviter_name = inviter.getProperty('fullname',
                                                   invitation.inviter)
            else:
                inviter_name = invitation.inviter

            item = dict(name=invitation.email,
                        roles=get_friendly_role_names(
                            invitation.roles, self.context, self.request),
                        inherited_roles=[],
                        inviter=inviter_name,
                        readonly=not member.getId() == invitation.inviter,
                        type_='invitations',
                        iid=invitation.iid)
            invitations.append(item)

        return invitations

    def get_users(self):
        result = self.get_participants() + self.get_pending_invitations()
        result.sort(key=lambda x: x['name'].lower())
        return result

    def hide_cancel_button(self):
        return not(self.has_participation_support())

    def has_participation_support(self):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        allow_invitation = config.allow_invite_users or config.allow_invite_email
        return IParticipationSupport.providedBy(
            self.context) and allow_invitation



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.invitation import Invitation
from ftw.participation.tests import layer
from ftw.testing.mailing import Mailing
from plone.app.testing import login
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zExceptions import Redirect
from zope.component import getUtility
from zope.interface import alsoProvides
import email


class TestAcceptInvitation(TestCase):

    layer = layer.FTW_PARTICIPATION_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.folder = create(Builder('folder'))
        alsoProvides(self.folder, IParticipationSupport)

        create(Builder('user').named('Felix', 'Leiter'))
        create(Builder('user').named('James', 'Bond'))

        Mailing(self.layer['portal']).set_up()

        self.portal.manage_changeProperties(
            {'email_from_name': 'Plone Admin',
             'email_from_address': 'plone@plone.local'})


    def tearDown(self):
        Mailing(self.layer['portal']).tear_down()

    def test_accept_confirmation_mail_encodings(self):
        invitation = Invitation(target=self.folder,
                                email='felix@leiter.com',
                                inviter='james.bond',
                                roles=['Reader'])

        login(self.portal, 'felix.leiter')
        view = self.portal.restrictedTraverse('accept_invitation')
        view(iid=invitation.iid)
        message = email.message_from_string(Mailing(self.portal).pop())

        self.assertItemsEqual(['text/plain; charset="utf-8"',
                               'text/html; charset="utf-8"'],
                              [payload.get('Content-Type')
                               for payload in message.get_payload()])

        self.assertEquals('=?utf-8?q?Bond_James?= <james@bond.com>',
                          message.get('To'))

        self.assertEquals('=?utf-8?q?Plone_Admin?= <plone@plone.local>',
                          message.get('From'))

        self.assertRegexpMatches(
            message.get('Subject'),
            r'^=\?utf-?8\?q\?The_Invitation_to_')

    def test_accepting_sets_roles_on_context(self):
        login(self.portal, 'james.bond')
        invitation = Invitation(target=self.folder,
                                email='felix@leiter.com',
                                inviter='james.bond',
                                roles=['Reader', 'Contributor'])

        login(self.portal, 'felix.leiter')
        view = self.portal.restrictedTraverse('accept_invitation')
        view(iid=invitation.iid)

        self.assertItemsEqual(
            ['Reader', 'Contributor'],
            dict(self.folder.get_local_roles()).get('felix.leiter'))

    def test_Reader_role_is_added_by_default(self):
        login(self.portal, 'james.bond')
        invitation = Invitation(target=self.folder,
                                email='felix@leiter.com',
                                inviter='james.bond',
                                roles=['Contributor'])

        login(self.portal, 'felix.leiter')
        view = self.portal.restrictedTraverse('accept_invitation')
        view(iid=invitation.iid)

        self.assertItemsEqual(
            ['Reader', 'Contributor'],
            dict(self.folder.get_local_roles()).get('felix.leiter'))

    def test_Reader_role_is_not_added_when_multiple_roles_disabled(self):
        # When allow_multiple_roles is disabled (default is enabled), we
        # only want to have one single role set.
        # In this case the default Reader role should not be appended,
        # otherwise we would have multiple roles against the configuration,
        # producing problems such as when changing roles with
        # radio buttons.

        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_multiple_roles = False

        login(self.portal, 'james.bond')
        invitation = Invitation(target=self.folder,
                                email='felix@leiter.com',
                                inviter='james.bond',
                                roles=['Contributor'])

        login(self.portal, 'felix.leiter')
        view = self.portal.restrictedTraverse('accept_invitation')
        view(iid=invitation.iid)

        self.assertItemsEqual(
            ['Contributor'],
            dict(self.folder.get_local_roles()).get('felix.leiter'))

    def test_accept_nonexisting_invitiation_is_aborted(self):
        view = self.portal.restrictedTraverse('accept_invitation')
        with self.assertRaises(Redirect) as cm:
            view(iid=999)

        self.assertEquals(self.portal.absolute_url(),
                          cm.exception.args[0])



from ftw.testbrowser import browser


def roles_input_types():
    """Returns the type of input fields used for role selection.
    """
    label = browser.xpath('//label[normalize-space(text())="Roles"]').first
    inputs = label.parent('div.field').css('input')
    types = map(lambda node: node.attrib['type'], inputs)
    return list(set(types) - set(['hidden']))



from Products.CMFCore.utils import getToolByName
from ftw.participation.interfaces import IInvitation, IInvitationStorage
from persistent import Persistent
from plone.uuid.interfaces import IUUID
from zope.component.hooks import getSite
from zope.interface import implements


class Invitation(Persistent):
    """The `Invitation` object is used for storing who has invited who for
    which context.

    """

    implements(IInvitation)

    def __init__(self, target, email, inviter, roles):
        """Creates a new `Invitation` on the `target` for `email`. The
        invitation was created by `inviter` (userid). Roles are the given
        additional roles for the user (email)
        """
        # get the storage
        storage = IInvitationStorage(target)
        # set the attributes
        self.email = email
        self.inviter = inviter
        self.set_target(target)
        self.roles = roles
        # generate the id, which sets it to self.iid
        storage.generate_iid_for(self)
        # register the invitation
        storage.add_invitation(self)

    def set_target(self, obj):
        """Sets the target to the given `obj`. Internally we store UIDs.
        """
        self._target = IUUID(obj)

    def get_target(self):
        """Returns the currently stored target or `None`
        """
        if '_target' not in dir(self):
            return None
        site = getSite()
        catalog = getToolByName(site, 'portal_catalog')

        result = catalog.unrestrictedSearchResults(UID=self._target)
        if len(result) != 1:
            return None
        else:
            brain, = result
            return site.unrestrictedTraverse(brain.getPath())



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.invitation import Invitation
from ftw.participation.tests.layer import FTW_PARTICIPATION_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from Products.CMFCore.utils import getToolByName
from unittest import TestCase
from zExceptions import Forbidden
from zope.interface import alsoProvides


class TestParticipation(TestCase):

    layer = FTW_PARTICIPATION_INTEGRATION_TESTING

    def setUp(self):
        super(TestParticipation, self).setUp()

        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        login(self.portal, TEST_USER_NAME)

        self.demo_folder = self.portal.get(self.portal.invokeFactory(
            'Folder', 'demo-folder'))
        alsoProvides(self.demo_folder, IParticipationSupport)

        self.view = self.demo_folder.restrictedTraverse('@@participants')
        self.layer['request'].set('ACTUAL_URL', '/'.join((
            self.demo_folder.absolute_url(), '@@participants')))

    def test_view_available(self):
        self.assertIsNotNone(self.view, 'Participants view is not available')

    def test_participants(self):
        regtool = getToolByName(self.portal, 'portal_registration')
        regtool.addMember('usera', 'secret',
                          properties={'username': 'usera',
                                      'fullname': 'Usera',
                                      'email': 'user@email.com'})

        self.portal.portal_membership.setLocalRoles(
            obj=self.demo_folder,
            member_ids=['usera', ],
            member_role="Reader",
            reindex=True)

        expect = [{'name': TEST_USER_ID,
                   'userid': TEST_USER_ID,
                   'readonly': True,
                   'roles': ['Owner'],
                   'inherited_roles': [],
                   'type_': 'userids'},
                  {'name': 'Usera (user@email.com)',
                   'userid': 'usera',
                   'inherited_roles': [],
                   'readonly': False,
                   'roles': [u'Can view'],
                   'type_': 'userids'}, ]

        self.assertEquals(expect, self.view.get_participants())

    def test_get_pending_invitations(self):
        invitation = Invitation(target=self.demo_folder,
                                email='user@email.com',
                                inviter=TEST_USER_NAME,
                                roles=['Reader'])

        expect = [dict(name='user@email.com',
                            roles=[u'Can view'],
                            inherited_roles=[],
                            inviter=invitation.inviter,
                            type_='invitations',
                            iid=invitation.iid,
                            readonly=True)]

        self.assertEquals(expect, self.view.get_pending_invitations())

    def test_sorted_result(self):
        regtool = getToolByName(self.portal, 'portal_registration')
        regtool.addMember('usera', 'secret',
                          properties={'username': 'usera',
                                      'fullname': '\xc3\x9csera',
                                      'email': 'user@email.com'})

        regtool.addMember('inviter', 'secret',
                          properties={'username': 'inviter',
                                      'fullname': 'invit\xc3\xb6r',
                                      'email': 'user@email.com'})

        self.demo_folder.manage_setLocalRoles('inviter', ['Owner'])

        Invitation(
            target=self.demo_folder,
            email='user@email.com',
            inviter=TEST_USER_NAME,
            roles=['Reader'])

        self.assertTrue(self.view.get_users())

    def test_readonly_user_cannot_remove_itself(self):
        self.assertTrue(self.view.get_participants()[0]['readonly'],
                        'The user should not be able to remove itself')

    def test_readonly_if_user_is_owner(self):
        user = create(Builder('user').with_roles('Owner', on=self.demo_folder))

        self.assertTrue(self.view.cannot_remove_user(user.getId()),
                        'It should not be possible to remove a owner')

    def test_not_readonly_if_only_local_roles_exists_and_no_user(self):
        userid = 'dummyid'
        self.demo_folder.manage_setLocalRoles(userid, ['Reader'])

        self.assertFalse(self.view.cannot_remove_user(userid),
                         'It should be possible to remove local roles if the '
                         'user does not longer exists')

    def test_readonly_if_not_inviter(self):
        Invitation(
            target=self.demo_folder,
            email='user@email.com',
            inviter='dummyuser',
            roles=['Reader'])

        self.assertTrue(self.view.get_pending_invitations(),
                        'It should no be possible to remove invitations if '
                        'the current user is not the inviter')

    def test_remove_user(self):
        regtool = getToolByName(self.portal, 'portal_registration')
        regtool.addMember('usera', 'secret',
                          properties={'username': 'usera',
                                      'fullname': 'Usera',
                                      'email': 'user@email.com'})

        self.portal.portal_membership.setLocalRoles(
            obj=self.demo_folder,
            member_ids=['usera', ],
            member_role="Reader",
            reindex=True)

        assert len(self.view.get_participants()) == 2, 'Expect 2 participants'

        form = {'userids': ['usera'],
                'form.delete': True}

        self.view.request.form = form
        self.view()

        self.assertEquals(1,
                          len(self.view.get_participants()),
                          'Expect only one participant the owner')

    def test_remove_user_forbidden(self):
        regtool = getToolByName(self.portal, 'portal_registration')
        regtool.addMember('usera', 'secret',
                          properties={'username': 'usera',
                                      'fullname': 'Usera',
                                      'email': 'user@email.com'})

        self.portal.portal_membership.setLocalRoles(
            obj=self.demo_folder,
            member_ids=['usera', ],
            member_role="Reader",
            reindex=True)

        logout()
        login(self.portal, 'usera')

        assert len(self.view.get_participants()) == 2, 'Expect 2 participants'

        form = {'userids': ['usera'],
                'form.delete': True}
        self.view.request.form = form

        self.assertRaises(Forbidden, self.view, *[])

    def test_remove_invitation(self):
        invitation = Invitation(target=self.demo_folder,
                                email='user@email.com',
                                inviter=TEST_USER_ID,
                                roles=['Reader'])

        assert len(self.view.get_pending_invitations()) == 1

        form = {'invitations': [invitation.iid],
                'form.delete': True}

        self.view.request.form = form
        self.view()

        self.assertEquals(0,
                          len(self.view.get_pending_invitations()),
                          'Expect no invitation')

    def test_remove_invitation_forbidden(self):
        invitation = Invitation(target=self.demo_folder,
                                email='user@email.com',
                                inviter='dummyuser',
                                roles=['Reader'])

        assert len(self.view.get_pending_invitations()) == 1

        form = {'invitations': [invitation.iid],
                'form.delete': True}

        self.view.request.form = form
        self.assertRaises(Forbidden, self.view, *[])

        self.assertEquals(1,
                          len(self.view.get_participants()),
                          'Expect one invitation')

    def test_participants_inherited_roles(self):

        folder = create(Builder('folder')
                        .titled('Folder'))

        subfolder = create(Builder('folder')
                           .within(folder)
                           .titled('Subfolder')
                           .providing(IParticipationSupport))

        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Reader', on=folder)
               .with_roles('Contributor', on=subfolder))

        expect = [{'name': u'Boss Hugo (hugo@boss.com)',
                   'userid': 'hugo.boss',
                   'readonly': False,
                   'roles': [u'Can add', u'Can view'],
                   'inherited_roles': [u'Can view'],
                   'type_': 'userids'},
                  {'name': unicode(TEST_USER_ID),
                   'userid': TEST_USER_ID,
                   'readonly': True,
                   'roles': ['Owner'],
                   'inherited_roles': [],
                   'type_': 'userids'}, ]

        view = subfolder.restrictedTraverse('@@participants')
        self.maxDiff = None
        self.assertEquals(expect, view.get_participants())

    def test_participants_inherited_roles_not_acquired(self):

        folder = create(Builder('folder')
                        .titled('Folder'))

        subfolder = create(Builder('folder')
                           .within(folder)
                           .titled('Subfolder')
                           .providing(IParticipationSupport))
        setattr(subfolder, '__ac_local_roles_block__', True)

        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Reader', on=folder)
               .with_roles('Contributor', on=subfolder))

        expect = [{'name': u'Boss Hugo (hugo@boss.com)',
                   'userid': 'hugo.boss',
                   'readonly': False,
                   'roles': [u'Can add'],
                   'inherited_roles': [],
                   'type_': 'userids'},
                  {'name': unicode(TEST_USER_ID),
                   'userid': TEST_USER_ID,
                   'readonly': True,
                   'roles': ['Owner'],
                   'inherited_roles': [],
                   'type_': 'userids'}, ]

        view = subfolder.restrictedTraverse('@@participants')
        self.maxDiff = None
        self.assertEquals(expect, view.get_participants())

    def test_participants_show_only_inherited_roles(self):

        folder = create(Builder('folder')
                        .titled('Folder'))

        subfolder = create(Builder('folder')
                           .within(folder)
                           .titled('Subfolder')
                           .providing(IParticipationSupport))

        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Contributor', on=folder))

        expect = [{'name': u'Boss Hugo (hugo@boss.com)',
                   'userid': 'hugo.boss',
                   'readonly': False,
                   'roles': [u'Can add'],
                   'inherited_roles': [u'Can add'],
                   'type_': 'userids'},
                  {'name': unicode(TEST_USER_ID),
                   'userid': TEST_USER_ID,
                   'readonly': True,
                   'roles': ['Owner'],
                   'inherited_roles': [],
                   'type_': 'userids'}, ]

        view = subfolder.restrictedTraverse('@@participants')
        self.maxDiff = None
        self.assertEquals(expect, view.get_participants())

    def test_participants_do_not_show_inherited_owner_role(self):

        folder = create(Builder('folder')
                        .titled('Folder'))

        subfolder = create(Builder('folder')
                           .within(folder)
                           .titled('Subfolder')
                           .providing(IParticipationSupport))
        del subfolder.__ac_local_roles__[TEST_USER_ID]
        subfolder._p_changed=True

        subfolder.manage_setLocalRoles(TEST_USER_ID, ['Editor'])

        expect = [{'name': unicode(TEST_USER_ID),
                   'userid': TEST_USER_ID,
                   'readonly': True,
                   'roles': ['Can edit'],
                   'inherited_roles': [],
                   'type_': 'userids'}, ]

        view = subfolder.restrictedTraverse('@@participants')
        self.maxDiff = None
        self.assertEquals(expect, view.get_participants())

    def test_exclude_inherited_users_without_roles(self):

        folder = create(Builder('folder')
                        .titled('Folder'))

        subfolder = create(Builder('folder')
                           .within(folder)
                           .titled('Subfolder')
                           .providing(IParticipationSupport))
        del subfolder.__ac_local_roles__[TEST_USER_ID]
        subfolder._p_changed=True

        expect = []
        view = subfolder.restrictedTraverse('@@participants')
        self.maxDiff = None
        self.assertEquals(expect, view.get_participants())



from ftw.participation import _
from ftw.participation.events import RolesChangedEvent
from ftw.participation.interfaces import IParticipationRegistry
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.button import buttonAndHandler
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import HIDDEN_MODE
from zExceptions import BadRequest
from zope import schema
from zope.component import getUtility
from zope.event import notify
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory


def get_user_local_roles(memberid, context):
    member = getToolByName(
        context,
        'portal_membership').getMemberById(memberid)

    if not member:
        raise BadRequest('No valid member id.')

    local_roles = dict(context.get_local_roles())
    return list(local_roles.get(member.getId(), []))


def extract_roles(context):
    factory = getUtility(IVocabularyFactory,
                         name='ftw.participation.roles')
    return [term.token for term in factory(context)]


class IChangeRoleSchema(Interface):

    roles = schema.List(
        title=_(u'label_roles', default=u'Roles'),
        value_type=schema.Choice(
            vocabulary=u'ftw.participation.roles'),
        required=False)

    memberid = schema.TextLine()


class ChangeRolesForm(Form):

    label = _(u'label_changerole_form', default=u'Change roles')

    ignoreContext = True
    fields = Fields(IChangeRoleSchema)

    allow_prefill_from_GET_request = True

    def update(self):
        memberid = self.request.form.get('form.widgets.memberid')
        if not memberid:
            raise BadRequest('No member given')

        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        if config.allow_multiple_roles:
            self.fields['roles'].widgetFactory = CheckBoxFieldWidget
        else:
            self.fields['roles'].widgetFactory = RadioFieldWidget

        if not self.request.form.get('form.widgets.roles-empty-marker'):
            default_roles = set(
                get_user_local_roles(memberid, self.context)).intersection(set(
                extract_roles(self.context)))
            self.request.form['form.widgets.roles'] = list(default_roles)

        super(ChangeRolesForm, self).update()

    def updateWidgets(self):
        super(ChangeRolesForm, self).updateWidgets()

        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        roles_widget = self.widgets['roles']
        if not config.allow_multiple_roles:
            roles_widget.required = True

        memberid_widget = self.widgets['memberid']
        memberid_widget.mode = HIDDEN_MODE

    @buttonAndHandler(_(u'button_save', default=u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()

        if errors:
            return

        data['roles'] = map(lambda role: role.encode('utf-8'), data['roles'])
        data['memberid'] = data['memberid'].encode('utf-8')

        notify(RolesChangedEvent(
            self.context,
            userid=data['memberid'],
            old_roles=get_user_local_roles(data['memberid'], self.context),
            new_roles=data['roles']))

        self.save_local_roles(data)
        return self.redirect()

    @buttonAndHandler(_(u'button_cancel', default=u'Cancel'))
    def handle_cancel(self, action):
        return self.redirect()

    def redirect(self):
        url = '{0}/@@participants'.format(self.context.absolute_url())
        return self.request.RESPONSE.redirect(url)

    def save_local_roles(self, data):
        # Remove all managed roles
        user_roles = get_user_local_roles(data['memberid'], self.context)
        managed_roles = extract_roles(self.context)
        filtered = filter(lambda role: role not in managed_roles, user_roles)

        # Append new roles
        filtered.extend(data['roles'])
        self.context.manage_setLocalRoles(data['memberid'], filtered)
        self.context.reindexObjectSecurity()
        msg = _(u'message_roles_changes', default=u'Changed roles')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')



from ftw.testbrowser import browser
from operator import itemgetter


def table():
    return browser.css('#content table.listing').first.dicts()


def users_column():
    return map(itemgetter('User'), table())


def checkboxes_visible():
    form = browser.forms['manage-participants']
    return 'userids:list' in form.values.keys()



from ftw.participation import _
from ftw.participation.browser.participants import get_friendly_role_names
from zope.i18n import translate


def translate_and_join_roles(roles, context, request):
    roles = get_friendly_role_names(roles, context, request)
    and_ = translate(_(' and '), context=request)
    return ', '.join(roles[:-2] + [and_.join(roles[-2:])])



from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.tests.layer import FTW_PARTICIPATION_INTEGRATION_TESTING
from plone.app.workflow.interfaces import ISharingPageRole
from plone.app.workflow.permissions import DelegateContributorRole
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.component import getSiteManager
from zope.component import getUtility
from zope.i18nmessageid import Message
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory


def as_dict(vocabulary):
    """Converts a simple vocabulary to a dict, using the tokens as key.
    """
    return dict([(term.token, term.title) for term in vocabulary])


class Tester(object):
    implements(ISharingPageRole)

    title = 'Can test'


class TestLocalRolesForDisplay(TestCase):

    layer = FTW_PARTICIPATION_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_Reviewer_role_is_excluded_by_default(self):
        self.assertNotIn('Reviewer',
                         as_dict(self.factory(self.portal)))

    def test_Reviewer_role_can_be_included_by_resetting_filters(self):
        self.assertIn('Reviewer',
                      as_dict(self.factory(self.portal, role_filter=[])))

    def test_Reader_role_is_excluded_by_default(self):
        self.assertNotIn('Reader',
                         as_dict(self.factory(self.portal)))

    def test_filter_reader_role_excludes_Reader_role(self):
        self.assertIn('Reader',
                      as_dict(self.factory(self.portal, filter_reader_role=False)))

    def test_allow_multiple_roles_option_required_for_filter_reader_role(self):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_multiple_roles = False

        self.assertIn('Reader',
                      as_dict(self.factory(self.portal, filter_reader_role=True)))

    def test_role_permissions_are_checked(self):
        self.assertIn('Contributor',
                      as_dict(self.factory(self.portal)))

        self.portal.manage_permission(DelegateContributorRole, roles=[], acquire=False)
        self.assertNotIn('Contributor',
                         as_dict(self.factory(self.portal)))

    def test_titles_are_translation_messages(self):
        contributor = as_dict(self.factory(self.portal))['Contributor']
        self.assertEquals(Message, type(contributor))
        self.assertEquals('Can add', contributor.default)

    def test_roles_without_permission_are_listed(self):
        getSiteManager().registerUtility(provided=ISharingPageRole,
                                         factory=Tester,
                                         name='Tester')
        self.assertIn('Tester',
                      as_dict(self.factory(self.portal)))

    @property
    def factory(self):
        return getUtility(IVocabularyFactory, name='ftw.participation.roles')



from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from ftw.participation import _
from ftw.participation.events import InvitationCreatedEvent
from ftw.participation.interfaces import IParticipationQuotaHelper
from ftw.participation.interfaces import IParticipationQuotaSupport
from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.invitation import Invitation
from plone import api
from plone.formwidget.autocomplete.widget import AutocompleteMultiFieldWidget
from plone.registry.interfaces import IRegistry
from plone.z3cform.layout import wrap_form
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.button import buttonAndHandler
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.util import getSpecification
from z3c.form.validator import InvariantsValidator
from z3c.form.validator import SimpleFieldValidator
from z3c.form.validator import WidgetsValidatorDiscriminators
from z3c.form.validator import WidgetValidatorDiscriminators
from zExceptions import NotFound
from zope import schema
from zope.component import provideAdapter, getUtility
from zope.event import notify
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import Invalid
import os.path
import re


class IInviteSchema(Interface):
    """Schema interface for invite form
    """

    users = schema.List(
        title=_(u'label_users', default=u'Users'),
        description=_(u'help_users',
                      default=u'Select users to invite.'),
        value_type=schema.Choice(
            vocabulary=u'ftw.participation.users'),
        required=False)

    addresses = schema.Text(
        title=_(u'label_addresses', default=u'E-Mail Addresses'),
        description=_(u'help_addresses',
                      default=u'Enter one e-mail address per line'),
        required=False)

    roles = schema.List(
        title=_(u'label_roles', default=u'Roles'),
        value_type=schema.Choice(
            vocabulary=u'ftw.participation.roles'),
        required=False)

    comment = schema.Text(
        title=_(u'label_comment', default=u'Comment'),
        description=_(u'help_comment',
                      default=u'Enter a comment which will be contained '
                      'in the invitaiton E-Mail'),
        required=False,
        )


class AddressesValidator(SimpleFieldValidator):
    """Validator for validating the e-mail addresses field
    """

    MAIL_EXPRESSION = r"^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=" +\
        "?^_`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?" +\
        "\.)+[a-z]{2,6}|([0-9]{1,3}\.){3}[0-9]{1,3})$"

    def __init__(self, *args, **kwargs):
        super(AddressesValidator, self).__init__(*args, **kwargs)
        self.email_expression = re.compile(AddressesValidator.MAIL_EXPRESSION,
                                           re.IGNORECASE)

    def validate(self, value):
        """Validates the `value`, expects a list of carriage-return-separated
        email addresses.

        """

        super(AddressesValidator, self).validate(value)
        if value:
            addresses = value.strip().split('\n')
            self._validate_addresses(addresses)

    def _validate_addresses(self, addresses):
        """E-Mail address validation
        """

        for addr in addresses:
            addr = addr.strip()
            if not self.email_expression.match(addr):
                msg = _(u'error_invalid_addresses',
                        default=u'At least one of the defined addresses '
                        'are not valid.')
                raise Invalid(msg)


WidgetValidatorDiscriminators(AddressesValidator,
                              field=IInviteSchema['addresses'])
provideAdapter(AddressesValidator)


class NumberOfAdressesAndUsersValidator(InvariantsValidator):

    def validate(self, data):
        errors = ()
        invitations_nr = (data.get('users') and len(data.get('users')) or 0) +\
            (data.get('addresses') and len(data.get('addresses')) or 0)

        if invitations_nr == 0:
            # at least one invitation required
            registry = getUtility(IRegistry)
            config = registry.forInterface(IParticipationRegistry)
            if config.allow_invite_email and config.allow_invite_users:
                errors += (Invalid(_(u'Select at least one user or enter at '
                                     'least one e-mail address')),)
            elif config.allow_invite_email:
                errors += (Invalid(_(u'Enter at least one e-mail address.')),)
            elif config.allow_invite_users:
                errors += (Invalid(_(u'Select at least one user.')),)

        if IParticipationQuotaSupport.providedBy(self.context):
            # check maximum participants when quota enabled
            quota_support = IParticipationQuotaHelper(self.context)
            allowed = quota_support.allowed_number_of_invitations()
            if invitations_nr > allowed:
                if allowed <= 0:
                    msg = _(u'error_participation_quota_reached',
                            default=u'The participation quota is reached, '
                            'you cannot add any further participants.')
                else:
                    msg = _('error_too_many_participants',
                            default=u'You cannot invite so many participants '
                            'any more. Can only add ${num} more participants.',
                            mapping={'num': allowed})
                raise Invalid(msg)

        return errors

WidgetsValidatorDiscriminators(NumberOfAdressesAndUsersValidator,
                               schema=getSpecification(IInviteSchema,
                                                       force=True))
provideAdapter(NumberOfAdressesAndUsersValidator)


class RolesValidator(SimpleFieldValidator):
    """Validator for validating the e-mail addresses field
    """

    def validate(self, roles):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        if not config.allow_multiple_roles and not roles:
            msg = _(u'error_no_roles', default=u'Please select a role')
            raise Invalid(msg)

WidgetValidatorDiscriminators(RolesValidator,
                               field=IInviteSchema['roles'])
provideAdapter(RolesValidator)


class InviteForm(Form):
    label = _(u'label_invite_participants', default=u'Invite Participants')

    description = _(u'text_invite_new',
                    default=u'Use this form to invide new user')

    ignoreContext = True
    fields = Fields(IInviteSchema)
    fields['users'].widgetFactory = AutocompleteMultiFieldWidget

    def update(self):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        if not (config.allow_invite_email or config.allow_invite_users):
            raise NotFound

        if config.allow_multiple_roles:
            self.fields['roles'].widgetFactory = CheckBoxFieldWidget
        else:
            self.fields['roles'].widgetFactory = RadioFieldWidget

        super(InviteForm, self).update()

    def updateWidgets(self):
        super(InviteForm, self).updateWidgets()
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        if not config.allow_invite_email:
            # disable address field
            del self.widgets['addresses']
            self.widgets['users'].required = True

        if not config.allow_invite_users:
            # disable users field
            del self.widgets['users']
            self.widgets['addresses'].required = True

        if not config.allow_multiple_roles:
            self.widgets['roles'].required = True

    def updateActions(self):
        super(InviteForm, self).updateActions()
        self.actions['button_invite'].addClass("context")

    @buttonAndHandler(_(u'button_invite', default=u'Invite'))
    def handle_invite(self, action):
        """Invite the users: create Invitations and send email
        """
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        data, errors = self.extractData()
        if len(errors) == 0:
            # get inviter
            mtool = getToolByName(self.context, 'portal_membership')
            inviter = mtool.getAuthenticatedMember()

            # get all addresses
            addresses = []
            if config.allow_invite_email and \
                    data.get('addresses') and len(data.get('addresses')):
                for addr in data.get('addresses').split('\n'):
                    addresses.append(addr.strip())

            if config.allow_invite_users and data.get('users'):
                for username in data['users']:
                    member = api.user.get(username=username)
                    addresses.append(member.getProperty('email'))

            # get roles
            roles = data.get('roles', [])

            # handle every email seperate
            emails = []  # used for info msg
            for email in addresses:
                email = email.strip()
                if not email:
                    continue
                inv = Invitation(self.context, email, inviter.getId(), roles)
                self.send_invitation(inv, email, inviter, data.get('comment'))
                notify(InvitationCreatedEvent(self.context, inv,
                                              data.get('comment')))
                emails.append(email)

            # notify user
            emails.sort()
            msg = _(u'info_invitations_sent',
                    default=u'The invitation mails were sent to ${emails}.',
                    mapping={'emails': ', '.join(emails), })
            IStatusMessage(self.request).addStatusMessage(msg, type='info')
            return self.redirect()

    @buttonAndHandler(_(u'button_cancel', default=u'Cancel'))
    def handle_cancel(self, action):
        return self.redirect()

    def redirect(self):
        """Redirect back
        """

        url = self.context.absolute_url() + '/@@participants'
        return self.request.RESPONSE.redirect(url)

    def send_invitation(self, invitation, email, inviter, comment):
        """Send a invitation email to a user
        """

        properties = getUtility(IPropertiesTool)
        # prepare from address for header
        header_from = Header(properties.email_from_name,
                             'utf-8')
        header_from.append(u'<%s>' % properties.email_from_address.
                           decode('utf-8'),
                           'utf-8')
        # get subject
        header_subject = Header(unicode(self.get_subject()), 'utf-8')

        # prepare comment
        pttool = getToolByName(self.context, 'portal_transforms')
        html_comment = comment and pttool('text_to_html', comment) or ''

        # prepare options
        options = {
            'invitation': invitation,
            'email': email,
            'inviter': inviter,
            'inviter_name': inviter.getProperty('fullname',
                                                False) or inviter.getId(),
            'text_comment': comment,
            'html_comment': html_comment,
            'link_url': self.get_url(invitation),
            'site_title': self.context.portal_url.getPortalObject().Title(),
            }

        # get the body views
        html_view = self.context.unrestrictedTraverse('@@invitation_mail_html')
        text_view = self.context.unrestrictedTraverse('@@invitation_mail_text')
        # make the mail
        msg = MIMEMultipart('alternative')
        msg['Subject'] = header_subject
        msg['From'] = str(header_from)
        msg['To'] = email

        # render and embedd html / text
        text_body = text_view(**options).encode('utf-8')
        msg.attach(MIMEText(text_body, 'plain', 'utf-8'))
        html_body = html_view(**options).encode('utf-8')
        msg.attach(MIMEText(html_body, 'html', 'utf-8'))
        # send the mail
        mh = getToolByName(self.context, 'MailHost')
        mh.send(msg)

    def get_subject(self):
        """Returns the translated subject of the email.
        """

        context_title = self.context.pretty_title_or_id().decode('utf-8')
        # -- i18ndude hint --
        if 0:
            _(u'mail_invitation_subject',
              default=u'Invitation for paticipating in ${title}',
              mapping=dict(title=context_title))
        # / -- i18ndude hint --
        # do not use translation messages - translate directly
        return translate(u'mail_invitation_subject',
                         domain='ftw.participation',
                         context=self.request,
                         default=u'Invitation for paticipating in ${title}',
                         mapping=dict(title=context_title))

    def get_url(self, invitation):
        """Returns the URL which is embedded in the invitation email. Usually
        the links points to the invitation view and uses the iid of the
        invitation as parameter, so that the invitation will be marked as
        pending in the session.
        """

        return os.path.join(self.context.portal_url(),
                            '@@invitations?iid=%s' % invitation.iid)

InviteParticipantsView = wrap_form(InviteForm)



from Products.Five.browser import BrowserView
import os.path


class WelcomeInvitedView(BrowserView):
    """If a user is invited and clicks on the link in the e-mail, he is
    redirected to this view.

    """

    def __call__(self):
        """Just redirect to @@register by default. This view can be
        customized dependendig on your project..

        """
        url = os.path.join(self.context.portal_url(), '@@register')
        return self.request.RESPONSE.redirect(url)



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.tests.layer import FTW_PARTICIPATION_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase
import transaction


class TestChangeRoles(TestCase):

    layer = FTW_PARTICIPATION_FUNCTIONAL_TESTING

    def setUp(self):
        self.folder = create(Builder('folder')
                             .titled('The Folder')
                             .providing(IParticipationSupport))

    @browsing
    def test_change_local_roles_on_post(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', on=self.folder))

        data = {'form.widgets.memberid': john.getId()}
        # Passing "data" will cause a POST request.
        browser.login().visit(self.folder, view='change_roles', data=data)

        browser.fill(
            {'form.widgets.roles:list': ['Contributor', 'Editor']}).submit()
        self.assertEquals(
            '{0}/@@participants'.format(self.folder.absolute_url()),
            browser.url)

        browser.open(self.folder, view='@@participants')
        table = browser.css('table.listing').first.lists()
        self.assertEquals('Can add, Can edit, Can view', table[1][2])

    @browsing
    def test_change_local_roles_on_get(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', on=self.folder))

        # Pass the member id in a GET request.
        browser.login().open(
            self.folder.absolute_url() + '/change_roles?form.widgets.memberid=' + john.getId()
        )

        browser.fill(
            {'form.widgets.roles:list': ['Contributor', 'Editor']}).submit()
        self.assertEquals(
            '{0}/@@participants'.format(self.folder.absolute_url()),
            browser.url)

        browser.open(self.folder, view='@@participants')
        table = browser.css('table.listing').first.lists()
        self.assertEquals('Can add, Can edit, Can view', table[1][2])

    @browsing
    def test_form_with_default_value(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', 'Contributor', on=self.folder))

        data = {'form.widgets.memberid': john.getId()}
        browser.login().visit(self.folder, view='change_roles', data=data)

        self.assertEquals('Contributor',
                          browser.css('[checked="checked"]').first.value)

    @browsing
    def test_do_not_remove_unmanaged_roles(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', 'Contributor', 'Editor',
                                  on=self.folder))

        self.folder.manage_permission(
            'Sharing page: Delegate Reader role',
            roles=[],
            acquire=False)
        transaction.commit()

        data = {'form.widgets.memberid': john.getId()}
        browser.login().visit(self.folder, view='change_roles', data=data)
        browser.fill(
            {'form.widgets.roles:list': []}).submit()

        self.assertIn(('john.doe', ('Reader', )),
                      self.folder.get_local_roles(),
                      'john should be still Reader')

    @browsing
    def test_raise_bad_request_if_no_memberid_is_given(self, browser):
        with browser.expect_http_error(reason='Bad Request'):
            browser.login().visit(self.folder, view='change_roles')

    @browsing
    def test_raise_bad_request_if_no_valid_memberid_is_given(self, browser):
        with browser.expect_http_error(reason='Bad Request'):
            data = {'form.widgets.memberid': 'invalid_member_id'}
            browser.login().visit(self.folder, view='change_roles', data=data)

    @browsing
    def test_show_change_link_if_user_can_change_local_roles(self, browser):
        create(Builder('user')
               .named('John', 'Doe')
               .with_roles('Member')
               .with_roles('Reader', 'Contributor', 'Editor',
                           on=self.folder))

        browser.login().visit(self.folder, view='@@participants')

        self.assertEquals('change',
                          browser.css('table').first.lists()[1][-1])
        self.assertEquals('change',
                          browser.css('a.ChangeRoles').first.text,
                          'Expect a change link')

    @browsing
    def test_do_not_show_change_link(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', on=self.folder))

        browser.login(john.getId()).visit(self.folder, view='@@participants')
        self.assertFalse(browser.css('a.ChangeRoles'),
                         'Do not expect a change link.')

    @browsing
    def test_change_role_view_respects_the_delegate_permissions(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Reader', on=self.folder))

        # Remove delegate contributor role permission
        self.layer['portal'].manage_permission(
            'Sharing page: Delegate Contributor role',
            roles=[],
            acquire=False)
        transaction.commit()

        data = {'form.widgets.memberid': john.getId()}
        browser.login().visit(self.folder, view='change_roles', data=data)

        checkboxes = browser.css('#content input[type="checkbox"]')
        self.assertEquals(1, len(checkboxes), 'Expect only one checkbox')
        self.assertEquals('Editor', checkboxes.first.node.attrib['value'])

    @browsing
    def test_no_change_link_on_pending_invitations(self, browser):
        john = create(Builder('user'))
        hugo = create(Builder('user').named('hugo', 'b'))
        create(Builder('invitation')
               .inviting(john)
               .to(self.folder)
               .invited_by(hugo))

        browser.login().visit(self.folder, view='@@participants')
        table = browser.css('table').first.lists()

        self.assertEquals('Pending', table[1][-2])
        self.assertEquals('', table[1][-1])

    @browsing
    def test_user_cannot_change_its_own_roles(self, browser):
        john = create(Builder('user')
                      .named('John', 'Doe')
                      .with_roles('Member')
                      .with_roles('Manager', 'Editor', on=self.folder))

        browser.login(john.getId()).visit(self.folder, view="@@participants")

        self.assertEquals('', browser.css('table').first.lists()[1][-1])



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IInvitationStorage
from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.tests.layer import FTW_PARTICIPATION_FUNCTIONAL_TESTING
from ftw.participation.tests.pages import inviteform
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from ftw.testing.mailing import Mailing
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.component import getUtility
from zope.interface import alsoProvides
import transaction


class TestInviteForm(TestCase):

    layer = FTW_PARTICIPATION_FUNCTIONAL_TESTING

    def setUp(self):
        self.folder = create(Builder('folder').titled('F\xc3\xb6lder'))
        alsoProvides(self.folder, IParticipationSupport)
        self.portal = self.layer['portal']
        Mailing(self.portal).set_up()
        transaction.commit()

    def tearDown(self):
        Mailing(self.portal).tear_down()

    @browsing
    def test_multiple_roles_selectable(self, browser):
        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals(['checkbox'], inviteform.roles_input_types())

    @browsing
    def test_only_one_role_selectable_by_configuration_setting(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_multiple_roles = False
        transaction.commit()

        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals(['radio'], inviteform.roles_input_types())

    @browsing
    def test_inviting_a_user_by_email(self, browser):
        self.portal.portal_properties.email_from_name = 'M\xc3\xa4i Site'
        transaction.commit()
        self.assertEquals([], self.get_invitations(email='hugo@boss.com'))

        browser.login().visit(self.folder, view='invite_participants')
        browser.fill({'E-Mail Addresses': 'hugo@boss.com',
                      'Roles': ['Contributor'],
                      'Comment': u'Hi th\xf6re'})
        browser.find('Invite').click()
        statusmessages.assert_no_error_messages()
        statusmessages.assert_message(
            'The invitation mails were sent to hugo@boss.com.')

        self.assertEquals(1, len(self.get_invitations(email='hugo@boss.com')),
                          'Expected one invitation to be available for hugo@boss.com')
        invitation, = self.get_invitations(email='hugo@boss.com')
        self.assertEquals(['Contributor'], invitation.roles)
        mail = Mailing(self.portal).pop()
        self.assertIn('From: =?utf-8?q?M=C3=A4i_Site?= <test@localhost>', mail)
        self.assertIn('To: hugo@boss.com', mail)
        self.assertIn('Content-Type: text/plain; charset="utf-8"', mail)
        self.assertIn('Content-Type: text/html; charset="utf-8"', mail)
        self.assertIn('=?utf-8?q?Invitation_for_paticipating_in_F=C3=B6lder?=',
                      mail)
        self.assertIn('Hi th=C3=B6re', mail)


    @browsing
    def test_query_users(self, browser):
        create(Builder('user').named('Hugo', 'Boss'))
        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals([['hugo.boss', 'Boss Hugo']],
                          browser.find('Users').query('Hugo'))

    @browsing
    def test_inviting_a_user_by_userid(self, browser):
        self.portal.portal_properties.email_from_name = 'M\xc3\xa4i Site'
        user = create(Builder('user').named('H\xc3\xbcgo', 'Boss'))
        self.assertEquals([], self.get_invitations(user=user))

        browser.login().visit(self.folder, view='invite_participants')
        browser.fill({'Users': [user.getId()],
                      'Roles': ['Contributor'],
                      'Comment': u'Hi th\xf6re'})
        browser.find('Invite').click()
        statusmessages.assert_no_error_messages()
        statusmessages.assert_message(
            'The invitation mails were sent to hugo@boss.com.')

        self.assertEquals(1, len(self.get_invitations(email='hugo@boss.com')),
                          'Expected one invitation to be available for hugo@boss.com')
        invitation, = self.get_invitations(email='hugo@boss.com')
        self.assertEquals(['Contributor'], invitation.roles)
        mail = Mailing(self.portal).pop()
        self.assertIn('From: =?utf-8?q?M=C3=A4i_Site?= <test@localhost>', mail)
        self.assertIn('To: hugo@boss.com', mail)
        self.assertIn('Content-Type: text/plain; charset="utf-8"', mail)
        self.assertIn('Content-Type: text/html; charset="utf-8"', mail)
        self.assertIn('=?utf-8?q?Invitation_for_paticipating_in_F=C3=B6lder?=',
                      mail)
        self.assertIn('Hi th=C3=B6re', mail)

    @browsing
    def test_inviting_a_user_by_userid_with_a_different_login_name_is_possible(self, browser):
        user = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_userid('hugo.boss')
                      .with_email('hugo.boss@example.ch'))

        # Happens if the user changes the login-name trough the web in the zmi
        # or if he imported users from another source.
        api.portal.get_tool('acl_users').source_users.updateUser('hugo.boss', 'hugo.boss@example.ch')
        transaction.commit()

        self.assertEquals([], self.get_invitations(user=user))

        browser.login().visit(self.folder, view='invite_participants')
        browser.fill({'Users': [user.getId()], 'Roles': ['Contributor']})
        browser.find('Invite').click()

        statusmessages.assert_no_error_messages()
        statusmessages.assert_message(
            'The invitation mails were sent to hugo.boss@example.ch.')

        self.assertEquals(1, len(self.get_invitations(email='hugo.boss@example.ch')),
                          'Expected one invitation to be available for hugo@boss.com')

    @browsing
    def test_get_invitations_from_context_providing_inavigationroot(
            self,
            browser):
        self.portal.portal_properties.email_from_name = 'M\xc3\xa4i Site'
        transaction.commit()

        subfolder = create(Builder('folder')
                           .within(self.portal)
                           .providing(INavigationRoot))

        user = create(Builder('user').named('H\xc3\xbcgo', 'Boss'))
        browser.login().visit(self.folder, view='invite_participants')
        browser.fill({'Users': [user.getId()],
                      'Roles': ['Contributor'],
                      'Comment': u'Hi th\xf6re'})
        browser.find('Invite').click()

        browser.login().visit(subfolder, view='invitations')
        self.assertEquals(
            ['hugo@boss.com'],
            browser.css('.invited-users-listing tbody .inv-user').text)

    def get_invitations(self, email=None, user=None):
        assert email or user, '"email" or "user" argument required'
        assert not (email and user), 'only one orgument of "email" and "user" allowed'
        if user:
            email = user.getProperty('email')

        self.layer['request'].SESSION = {}
        storage = IInvitationStorage(self.layer['portal'])
        return storage.get_invitations_for_email(email)

    @browsing
    def test_invite_raises_notfound_if_participation_isnt_possible(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_invite_users = False
        config.allow_invite_email = False
        transaction.commit()

        with browser.expect_http_error(reason='Not Found'):
            browser.login().visit(self.folder, view='invite_participants')

    @browsing
    def test_invitation_only_by_users_is_possible(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_invite_users = True
        config.allow_invite_email = False
        transaction.commit()

        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals(
            'Invite participants',
            browser.css('.documentFirstHeading').first.text
        )

    @browsing
    def test_invitation_only_by_email_is_possible(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_invite_users = False
        config.allow_invite_email = True
        transaction.commit()

        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals(
            'Invite participants',
            browser.css('.documentFirstHeading').first.text
        )

    @browsing
    def test_invitation_by_users_and_email_is_possible(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_invite_users = True
        config.allow_invite_email = True
        transaction.commit()

        browser.login().visit(self.folder, view='invite_participants')
        self.assertEquals(
            'Invite participants',
            browser.css('.documentFirstHeading').first.text
        )



from ftw.upgrade import UpgradeStep


class UpgradeRegistry(UpgradeStep):
    """Add allow_multiple_roles entry to registry
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.participation.upgrades:1007')



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IInvitationAcceptedEvent
from ftw.participation.interfaces import IInvitationCreatedEvent
from ftw.participation.interfaces import IInvitationRejectedEvent
from ftw.participation.interfaces import IInvitationRetractedEvent
from ftw.participation.interfaces import ILocalRoleRemoved
from ftw.participation.interfaces import IParticipationEvent
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.interfaces import IRolesChangedEvent
from ftw.participation.invitation import Invitation
from ftw.participation.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import transaction


EVENTS = []

def record_event(event):
    EVENTS.append(event)


class TestEvents(FunctionalTestCase):

    def setUp(self):
        super(TestEvents, self).setUp()
        EVENTS[:] = []
        self.portal.getSiteManager().registerHandler(
            factory=record_event,
            required=(IParticipationEvent,))

    def tearDown(self):
        super(TestEvents, self).tearDown()
        self.portal.getSiteManager().unregisterHandler(
            factory=record_event,
            required=(IParticipationEvent,))

    @browsing
    def test_invitation_created_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        browser.login().open(folder, view='invite_participants')
        browser.fill({'E-Mail Addresses': 'hugo@boss.com',
                      'Roles': ['Contributor'],
                      'Comment': u'Hi th\xf6re'})
        browser.find('Invite').click()

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IInvitationCreatedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('hugo@boss.com', event.invitation.email)
        self.assertEquals(u'Hi th\xf6re', event.comment)

    @browsing
    def test_invitation_accepted_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named('John', 'Doe'))
        user = create(Builder('user').named('Hugo', 'Boss'))
        Invitation(target=folder,
                   email=user.getProperty('email'),
                   inviter=inviter.getId(),
                   roles=['Reader'])
        transaction.commit()

        browser.login(user).open(view='invitations')
        browser.click_on("Yes, I'd like to participate")

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IInvitationAcceptedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('hugo@boss.com', event.invitation.email)

    @browsing
    def test_invitation_rejected_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named('John', 'Doe'))
        user = create(Builder('user').named('Hugo', 'Boss'))
        Invitation(target=folder,
                   email=user.getProperty('email'),
                   inviter=inviter.getId(),
                   roles=['Reader'])
        transaction.commit()

        browser.login(user).open(view='invitations')
        browser.click_on("No, I reject the invitation")

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IInvitationRejectedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('hugo@boss.com', event.invitation.email)

    @browsing
    def test_invitation_retracted_event_on_invitations_view(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named('John', 'Doe'))
        invitation = Invitation(target=folder,
                                email='foo@bar.com',
                                inviter=inviter.getId(),
                                roles=['Reader'])
        transaction.commit()

        browser.login(inviter).open(view='invitations')
        browser.fill({'sent_invitations:list': invitation.iid})
        browser.click_on('Retract')

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IInvitationRetractedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('foo@bar.com', event.invitation.email)

    @browsing
    def test_invitation_retracted_event_on_participants_view(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named('John', 'Doe')
                         .with_roles('Manager', on=folder))
        invitation = Invitation(target=folder,
                                email='foo@bar.com',
                                inviter=inviter.getId(),
                                roles=['Reader'])
        transaction.commit()

        browser.login(inviter).open(folder, view='participants')
        browser.fill({'invitations:list': invitation.iid})
        browser.click_on('Delete Participants')

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IInvitationRetractedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('foo@bar.com', event.invitation.email)

    @browsing
    def test_roles_changed_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        create(Builder('user').named('John', 'Doe')
               .with_roles('Editor', on=folder))

        browser.login().open(folder, view='participants')
        browser.click_on('change')
        browser.fill({'Roles': ['Contributor']}).submit()

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(IRolesChangedEvent.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('john.doe', event.userid)
        self.assertEquals(['Editor'], event.old_roles)
        self.assertEquals(['Contributor'], event.new_roles)

    @browsing
    def test_local_role_removed(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        user = create(Builder('user').named('John', 'Doe')
                      .with_roles('Editor', on=folder))

        browser.login().open(folder, view='participants')
        browser.fill({'userids:list': [user.getId()]}) \
               .find('Delete Participants').click()

        self.assertEquals(1, len(EVENTS), 'Expected 1 event to be fired.')
        event, = EVENTS
        self.assertTrue(ILocalRoleRemoved.providedBy(event))
        self.assertEquals(folder.getPhysicalPath(),
                          event.object.getPhysicalPath())
        self.assertEquals('john.doe', event.userid)



from ftw.participation.tests.layer import FTW_PARTICIPATION_FUNCTIONAL_TESTING
from ftw.participation.tests.layer import TEST_USER_ID_2, TEST_USER_PW_2
from plone.testing.z2 import Browser
from unittest import TestCase
from ftw.participation.invitation import Invitation
import transaction


class TestInvitation(TestCase):

    layer = FTW_PARTICIPATION_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestInvitation, self).setUp()

        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.demo_folder = self.portal.get(self.portal.invokeFactory(
            'Folder', 'demo-folder'))
        transaction.commit()

        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def test_userid_differ_userlogin(self):
        member = self.portal.portal_membership.getMemberById(TEST_USER_ID_2)
        member.setMemberProperties({'email':'foo@bar.com'})
        inv1 = Invitation(target=self.demo_folder, email='foo@bar.com', inviter='admin',
                          roles=['Reader'])
        url = '%s/@@invitations?iid=%s' % (self.portal_url, inv1.iid)
        transaction.commit()
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_ID_2, TEST_USER_PW_2, ))

        self.browser.open(url)
        self.assertIn('Invitations', self.browser.contents)



from Products.CMFCore.utils import getToolByName
from datetime import datetime
from ftw.participation.interfaces import IInvitationStorage
from persistent.dict import PersistentDict
from persistent.list import PersistentList
from zope.annotation.interfaces import IAnnotations
from zope.component import adapts
from zope.interface import Interface
from zope.interface import implements
import AccessControl
from hashlib import md5


class InvitationStorage(object):
    """The invitation stores and manages the invitation. All modification in
    the list of invitations (appending new invitations, deleting existing
    invitations, etc.) should be done with through the invitation storage.

    The invitation storage is adapter adapting any context. The storage stores
    its data into annotations on the plone site root.
    """

    implements(IInvitationStorage)
    adapts(Interface)

    ANNOTATIONS_DATA_KEY = 'ftw.participation.storage : invitations'
    SESSION_PENDING_KEY = 'ftw.participation.storage : pending invitations'

    def __init__(self, context):
        """Adapts any context within a plone site
        """
        self.context = context
        portal_url = getToolByName(self.context, 'portal_url')
        self.portal = portal_url.getPortalObject()

    @property
    def portal_annotations(self):
        """The annotations of the plone site root
        """
        return IAnnotations(self.portal)

    @property
    def invitations(self):
        """A persistent dictionary of all invitations. The key is the email
        address of the invited user, the values are `PersistentList`s of
        `Invitations` of this user / email address.

        """
        cache_key = '_invitations_list'
        if cache_key not in dir(self):
            # didnt cache it yet, get it from the annotations (and cache it)
            if self.ANNOTATIONS_DATA_KEY not in self.portal_annotations:
                # dictionary does not exist in the annotations, create it
                self.portal_annotations[self.ANNOTATIONS_DATA_KEY] = \
                    PersistentDict()
            setattr(self, cache_key,
                    self.portal_annotations[self.ANNOTATIONS_DATA_KEY])
        return getattr(self, cache_key)

    def get_used_iids(self):
        """Returns a dict of all currently existing invitations with their id
        as key. This method is complex, do not use it often :)

        """
        data = {}
        for email, users_invitiations in self.invitations.items():
            for invitation in users_invitiations:
                data[invitation.iid] = invitation
        return data

    def generate_iid_for(self, invitation):
        """Generates a invitation id for the invitation and sets it (and
        returns it eventually).

        """
        used_iids = self.get_used_iids()
        # create a string as unique as possible, which will be hashed later
        # for that the info of the string are not decodable by the user
        base = '-'.join((
                str(datetime.now().isoformat()),
                invitation.email,
                invitation.inviter))
        counter = 1
        iid = None
        while not iid or iid in used_iids:
            iid = md5(base + str(counter)).hexdigest().strip()
            counter += 1
        # set the iid on the invitation
        invitation.iid = iid
        return iid

    def get_invitation_by_iid(self, iid, default=None):
        """Returns the invitation with the `iid` or `default` if there is none.
        """
        for users_invitiations in self.invitations.values():
            for invitation in users_invitiations:
                for invitation in users_invitiations:
                    if invitation.iid == iid:
                        return invitation
        return default

    def get_invitations_for_email(self, email=None, default=[]):
        """Returns a list of invitations for aspecific email address. If
        `email` is `None` the email is guessed by taking the email (or the
        userid) of the currently authenticated user. If there are no results,
        `default` is returend.
        """
        # guess the email if its `None`
        if email is None:
            mtool = getToolByName(self.context, 'portal_membership')
            member = mtool.getAuthenticatedMember()
            email = member.getProperty('email', member.getId())
        # move session-invitations - if necessary
        self.assign_pending_session_invitations()
        # the email is the key of the invitations data dict
        if email in self.invitations:
            return self.invitations.get(email)
        else:
            return default

    def get_invitations_invited_by(self, userid=None, default=[]):
        """Returns a list of invitations which are created by a specific user
        for any email address. If `userid` is `None` the userid of the
        authenticated member is taken. If therer are no results, `default`
        is returned.
        """
        # guess the `userid` if its `None`
        if userid is None:
            mtool = getToolByName(self.context, 'portal_membership')
            member = mtool.getAuthenticatedMember()
            userid = member.getId()
        # find all inivitations where the inviter is `userid`
        list_ = []
        for users_invitiations in self.invitations.values():
            for inv in users_invitiations:
                if inv.inviter == userid:
                    list_.append(inv)
        return list_ or default

    def get_invitations_for_context(self, context, default=[]):
        """Returns all invitations for a context. This method may be slow.
        """
        list_ = []
        for users_invitiations in self.invitations.values():
            for inv in users_invitiations:
                if inv.get_target() == context:
                    list_.append(inv)
        return list_ or default

    def get_pending_session_invitations(self, default=[]):
        """When the user clicks on the invitation in the email program the
        invitation is temporary stored in the session too (if the user is
        not logged in). In some situations (e.g. after registrering or logging
        in) the invitation (from the session) is moved to the user which the
        user is now logged in.
        This covers some special use cases, when as example the user does not
        use the email address, to which the invitation is sent for the user
        of the plone site. That's how invitations are reassigned to other
        users.

        This method returns a list of the current session-invitations or
        `default` if there is no list.
        """
        session = self.context.REQUEST.SESSION
        return session.get(self.SESSION_PENDING_KEY, default)

    def set_invitation_pending_for_session(self, invitation):
        """Sets the `invitation` as pending in the session.
        See the docstring of get_pending_session_invitations for further
        details.

        """
        session = self.context.REQUEST.SESSION
        if self.SESSION_PENDING_KEY not in session.keys():
            session[self.SESSION_PENDING_KEY] = PersistentList()
        session[self.SESSION_PENDING_KEY].append(invitation.iid)

    def assign_pending_session_invitations(self, email=None):
        """
        This method reassigns the session-invitations to the `email` (or the
        authenticated user if `email` is `None`) and removes them from the
        session.

        See the docstring of get_pending_session_invitations for futher
        details.
        """
        # guess the email, if it's not defined
        if email is None:
            mtool = getToolByName(self.context, 'portal_membership')
            member = mtool.getAuthenticatedMember()
            if member == AccessControl.SpecialUsers.nobody:
                # the user is not logged in and the `email` is not defined, so
                # we do not know to whom whe should assign the invitations
                return False
            # use the email (or - if not defined - the userd)
            email = member.getProperty('email', member.getId())
        # get the pending invitations from the session
        pending_invitations = self.get_pending_session_invitations()
        if pending_invitations and len(pending_invitations):
            for iid in pending_invitations[:]:
                inv = self.get_invitation_by_iid(iid)
                if inv:
                    self.reassign_invitation(inv, email)
            # remove the list from the session - all session-invitations are
            # dropped from the session
            del self.context.REQUEST.SESSION[self.SESSION_PENDING_KEY]

    def add_invitation(self, invitation):
        """Adds a invitation object to the storage.
        """
        # the key of the invitations dict is `invitaiton.email`, the value
        # is a `PersistentDict`
        if invitation.email not in self.invitations:
            self.invitations[invitation.email] = PersistentList()
        # check if its already added
        if invitation in self.invitations[invitation.email]:
            return False
        else:
            self.invitations[invitation.email].append(invitation)

    def remove_invitation(self, invitation):
        """Removes the `invitation` from the storage and destroys it
        """
        if invitation.email in self.invitations and \
                invitation in self.invitations[invitation.email]:
            # if the invitation is stored under invitation.email remove
            # it from there...
            self.invitations[invitation.email].remove(invitation)
            if len(self.invitations[invitation.email]) == 0:
                del self.invitations[invitation.email]
            return True
        else:
            # ... otherwise search it
            for email, users_invitiations in self.invitations.items():
                if invitation in users_invitiations:
                    users_invitiations.remove(invitation)
                    if len(users_invitiations) == 0:
                        del self.invitations[email]
                    return True
        return False

    def reassign_invitation(self, invitation, new_email):
        """Reassigns a invitation to another email address (`new_email`).
        """
        # remove invitation from storage
        self.remove_invitation(invitation)
        # change the email
        invitation.email = new_email
        # re-add it to the storage
        self.add_invitation(invitation)



from AccessControl import getSecurityManager
from ftw.builder import builder_registry
from ftw.participation.invitation import Invitation
import transaction


class InvitationBuilder(object):

    def __init__(self, session):
        self.session = session
        self.arguments = {
            'target': None,
            'email': None,
            'inviter': getSecurityManager().getUser().getId(),
            'roles': ['Reader']}

    def having(self, **kwargs):
        self.arguments.update(kwargs)
        return self

    def to(self, target):
        return self.having(target=target)

    def inviting(self, user):
        return self.having(email=user.getProperty('email'))

    def invited_by(self, user):
        return self.having(inviter=user.getId())

    def create(self, **kwargs):
        self.before_create()
        obj = Invitation(**self.arguments)
        self.after_create(obj)
        return obj

    def before_create(self):
        pass

    def after_create(self, obj):
        if self.session.auto_commit:
            transaction.commit()


builder_registry.register('invitation', InvitationBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.tests.layer import FTW_PARTICIPATION_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testing.mailing import Mailing
from plone.app.testing import login
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone import api
from unittest import TestCase
import transaction


class TestReminder(TestCase):

    layer = FTW_PARTICIPATION_FUNCTIONAL_TESTING

    def setUp(self):

        self.portal = self.layer['portal']
        Mailing(self.portal).set_up()
        transaction.commit()

        self.folder = create(Builder('folder')
                             .titled(u'The F\xf6lder')
                             .providing(IParticipationSupport))

        self.inviter = api.user.get(userid=TEST_USER_ID)
        self.grant('Manager')

    def grant(self, *roles):
        setRoles(self.portal, TEST_USER_ID, list(roles))
        transaction.commit()

    def tearDown(self):
        Mailing(self.layer['portal']).tear_down()

    @browsing
    def test_send_reminder_mail(self, browser):

        hugo = create(Builder('user')
                          .named(u'Hugo', u'B\xf6ss'))

        create(Builder('invitation')
               .inviting(hugo)
               .to(self.folder)
               .invited_by(self.inviter))

        browser.login()
        browser.visit(self.folder, view='participants')
        browser.fill({
            'invitations:list': True
        })
        browser.find('Resend Invitations').click()

        mail = Mailing(self.portal).pop()
        self.assertIn("hugo@boss.com", mail)
        self.assertIn("This is a reminder.", mail)

    @browsing
    def test_no_user_selected(self, browser):

        browser.login()
        browser.visit(self.folder, view='participants')

        browser.find('Resend Invitations').click()

        mails = Mailing(self.portal)
        self.assertEqual(len(mails.get_messages()), 0)



from Products.Archetypes.atapi import IntegerField, IntegerWidget
from Products.CMFCore.permissions import ManagePortal
from ftw.participation.interfaces import IParticipationQuotaSupport
from ftw.participation.interfaces import IParticipationQuotaHelper
from ftw.participation.interfaces import IInvitationStorage
from ftw.participation import _
from zope.interface import implements
from zope.component import adapts
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import ISchemaExtender


class ParticipationQuotaIntField(ExtensionField, IntegerField):
    """Integer field for setting the participation quota limit.
    """


class ParticipationQuotaExtender(object):
    """Extends the context with participation_quota_limit field using
    archetypes.schemaextender.

    """
    implements(ISchemaExtender)
    adapts(IParticipationQuotaSupport)

    fields = [
        ParticipationQuotaIntField(
            name='participation_limit_quota',
            schemata='quota',
            required=False,
            default=5,
            write_permission=ManagePortal,
            searchable=False,
            widget=IntegerWidget(
                label=_(u'label_participation_quota_limit',
                        default=u'Participation quota limit'),
                description=_(u'help_participation_quota_limit',
                              default=u''),
                size=20)),
        ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields


class ParticipationQuotaHelper(object):
    """Support methdos for participation quota
    """
    implements(IParticipationQuotaHelper)
    adapts(IParticipationQuotaSupport)

    def __init__(self, context):
        self.context = context

    def get_quota_limit(self):
        """Returns the quota limit for this context.
        """
        field = self.context.Schema().getField('participation_limit_quota')
        return field.get(self.context)

    def allowed_number_of_invitations(self):
        """Returns how many users can be invited on the context at the moment.
        Pending invitations are counted.

        """
        allowed = int(self.get_quota_limit())
        # subtract amount of local roles
        allowed -= len(self.context.get_local_roles())
        # subtract the amount of pending invitations
        storage = IInvitationStorage(self.context)
        allowed -= len(storage.get_invitations_for_context(self.context))
        return allowed

    def is_quota_exceeded(self):
        """Returns True if the participation quota is reached.
        """
        return self.allowed_number_of_invitations() > 0



from ftw.participation import interfaces
from ftw.participation.interfaces import IParticipationRegistry
from plone.registry.interfaces import IRegistry
from zope.component import adapts
from zope.component import getUtility
from zope.interface import Interface
from zope.interface import implements
import AccessControl


class DefaultParticipationSetter(object):
    """Sets up a participation on a object for a user.
    Discriminators:
    - context
    - request
    - invitation
    - member

    """

    adapts(interfaces.IParticipationSupport,
           Interface,
           interfaces.IInvitation,
           Interface)

    implements(interfaces.IParticipationSetter)

    def __init__(self, context, request, invitation, member):
        self.context = context
        self.request = request
        self.invitation = invitation
        self.member = member

    def __call__(self):
        """Participates the user
        """
        return self.participate_user()

    def participate_user(self):
        """Participates `self.user` on `self.context`.
        """
        local_roles = dict(self.context.get_local_roles())
        # get all current local roles of the user on this context
        user_roles = list(local_roles.get(self.member.getId(), []))
        user_roles.extend(self.roles())
        # make the roles unique
        user_roles = dict(zip(user_roles, user_roles)).keys()

        # Set the local roles with the security of the inviter. If
        # he has no longer permissions on this context this will
        # fail.
        _old_security_manager = AccessControl.getSecurityManager()
        _new_user = self.context.acl_users.getUserById(
            self.invitation.inviter)
        AccessControl.SecurityManagement.newSecurityManager(
            self.request, _new_user)
        try:
            self.context.manage_setLocalRoles(self.member.getId(),
                                              user_roles)
            self.context.reindexObjectSecurity()
        except:
            AccessControl.SecurityManagement.setSecurityManager(
                _old_security_manager)
            raise
        else:
            AccessControl.SecurityManagement.setSecurityManager(
                _old_security_manager)

    def roles(self):
        """List of roles to give the `self.user` on the `self.context`.
        Reader ist default, and must be set.
        """
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)

        if config.allow_multiple_roles:
            default_role = ['Reader', ]
        else:
            default_role = []

        if hasattr(self.invitation, 'roles'):
            return list(self.invitation.roles) + default_role
        return default_role



from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from ftw.participation.interfaces import IParticipationRegistry
from plone.app.workflow.interfaces import ISharingPageRole
from plone.registry.interfaces import IRegistry
from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class LocalRolesForDisplay(object):
    """Vocabulary of local roles"""

    implements(IVocabularyFactory)

    def __call__(self, context, role_filter=None, filter_reader_role=True):
        """Get a list of roles, similar like @@sharing"""

        if role_filter is None:
            role_filter = ['Reviewer', ]

        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        if filter_reader_role and config.allow_multiple_roles:
            role_filter = role_filter + ['Reader']

        roles = []
        for name, utility in self.get_role_utilities(context, role_filter):
            roles.append(SimpleTerm(name, name, utility.title))

        roles.sort(lambda x, y: cmp(x.title, y.title))
        return SimpleVocabulary(roles)

    def get_role_utilities(self, context, exclude):
        context = aq_inner(context)
        portal_membership = getToolByName(context, 'portal_membership')

        for name, utility in getUtilitiesFor(ISharingPageRole):
            if name in exclude:
                continue

            permission = getattr(utility, 'required_permission', None)
            if not permission:
                allowed = True
            else:
                allowed = portal_membership.checkPermission(
                    permission, context)

            if allowed:
                yield name, utility


LocalRolesForDisplayFactory = LocalRolesForDisplay()



from ftw.participation import _
from zope import schema
from zope.component.interfaces import IObjectEvent
from zope.interface import Attribute
from zope.interface import Interface


class IParticipationBrowserLayer(Interface):
    """Browser layer interface for ftw.participation
    """


class IParticipationRegistry(Interface):
    """plone.app.registry configuration interface
    """

    allow_invite_users = schema.Bool(
        title=_(u'label_allow_invite_users',
                default=u'Allow to invite users'),
        description=_(u'help_allow_invite_users',
                      default=u'Allow to search users from PAS in invitation '
                      'form.'),
        default=True)

    allow_invite_email = schema.Bool(
        title=_(u'label_allow_invite_email',
                default=u'Allow to invite participants by email'),
        description=_(u'help_allow_invite_email',
                      default=u'Allow to invite participants by typing the '
                      'e-mail address in the invitation form. It is '
                      'recommended to enable registriation in plone security'),
        default=True)

    allow_multiple_roles = schema.Bool(
        title=_(u'label_multiple_roles',
                default=u'Allow to give multiple roles'),
        description=_(u'help_multiple_roles',
                      default=u'Allow to pass multiple roles to a invited '
                               'user.'),
        default=True)


class IInvitation(Interface):
    """Marker interface for invitations
    """


class IInvitationStorage(Interface):
    """Adapter interface for managing and storing invitations
    """


class IParticipationSupport(Interface):
    """Marker interface for objects where other users can be invitation
    """


class IInvitationMailer(Interface):
    """Adapter interface for the invitation mailer
    """


class IParticipationSetter(Interface):
    """Adapter interface for the adapter which sets the permission or does
    other things for participating a user.
    Discriminators: context, request, invitation, member

    """


class IParticipationQuotaSupport(Interface):
    """This marker interface enables a participation quota on the provided
    context. A protected field for defining the quota limit is added to
    the schema.

    """


class IParticipationQuotaHelper(Interface):
    """Helper adapter for the participation quota.
    """


class IParticipationEvent(IObjectEvent):
    """Superclass of all ftw.participation events.
    """


class IInvitationEvent(IParticipationEvent):
    """An event related to an invitation.
    """

    invitation = Attribute('The invitation')


class IInvitationCreatedEvent(IInvitationEvent):
    """An invitation was created.
    """

    comment = Attribute('Comment')


class IInvitationAcceptedEvent(IInvitationEvent):
    """An invitation was accepted.
    """


class IInvitationRejectedEvent(IInvitationEvent):
    """An invitation was rejected.
    """


class IInvitationRetractedEvent(IInvitationEvent):
    """An invitation was retracted.
    """


class IRolesChangedEvent(IParticipationEvent):
    """The roles of a participant was changed.
    """

    userid = Attribute('userid of the user of whom the roles are changed')
    old_roles = Attribute('roles of the userid before it was changed')
    new_roles = Attribute('roles of the userid after it was changed')


class ILocalRoleRemoved(IParticipationEvent):
    """A local role was removed.
    """

    userid = Attribute('userid of the user who was removed')



from ftw.participation.activity.utils import translate_and_join_roles
from ftw.participation.tests import FunctionalTestCase


class TestInvitationRenderer(FunctionalTestCase):

    def test_translate_and_join_roles__1_role(self):
        self.assertEquals(
            'Can add',
            translate_and_join_roles(('Contributor',), None, None))

    def test_translate_and_join_roles__2_roles(self):
        self.assertEquals(
            'Can add and Can view',
            translate_and_join_roles(('Reader', 'Contributor'), None, None))

    def test_translate_and_join_roles__3_roles(self):
        self.assertEquals(
            'Can add, Can edit and Can view',
            translate_and_join_roles(('Reader', 'Editor', 'Contributor'), None, None))

    def test_translate_and_join_roles__4_roles(self):
        self.assertEquals(
            'Can add, Can edit and Can view',
            translate_and_join_roles(
                ('Reader', 'Editor', 'Contributor', 'Manager'), None, None))



from ftw.activity.tests.helpers import get_soup_activities
from ftw.activity.tests.pages import activity
from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.invitation import Invitation
from ftw.participation.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import transaction


class TestActivity(FunctionalTestCase):

    @browsing
    def test_invitation_created_activity(self, browser):
        self.grant('Manager')
        user = create(Builder('user').named(u'J\xf6hn', u'Doe')
                      .with_roles('Manager'))
        folder = create(Builder('folder').providing(IParticipationSupport))
        browser.login(user).open(folder, view='invite_participants')
        browser.fill({'E-Mail Addresses': 'hugo@boss.com',
                      'Roles': ['Contributor'],
                      'Comment': u'Hi th\xf6re'})
        browser.find('Invite').click()

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:invitation_created',
                 'actor': 'john.doe',
                 'path': '/plone/folder',
                 'invitation:inviter': u'john.doe',
                 'invitation:email': u'hugo@boss.com',
                 'invitation:roles': (u'Contributor',),
                 'invitation:comment': u'Hi th\xf6re',
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'invitation:inviter',
                                 'invitation:email',
                                 'invitation:roles',
                                 'invitation:comment')))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Invitation created now by Doe J\xf6hn',
             'title': ''},
            newest_event.infos())
        self.assertEquals(
            u'Doe J\xf6hn has invited hugo@boss.com as Can add.',
            newest_event.body_text)

    @browsing
    def test_invitation_accepted_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named(u'J\xf6hn', u'Doe'))
        user = create(Builder('user').named(u'Hugo', u'B\xf6ss'))
        Invitation(target=folder,
                   email=user.getProperty('email'),
                   inviter=inviter.getId(),
                   roles=['Reader'])
        transaction.commit()

        browser.login(user).open(view='invitations')
        browser.click_on("Yes, I'd like to participate")

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:invitation_accepted',
                 'actor': 'hugo.boss',
                 'path': '/plone/folder',
                 'invitation:inviter': 'john.doe',
                 'invitation:email': 'hugo@boss.com',
                 'invitation:roles': ('Reader',),
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'invitation:inviter',
                                 'invitation:email',
                                 'invitation:roles',)))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Invitation accepted now by B\xf6ss Hugo',
             'title': ''},
            newest_event.infos())

    @browsing
    def test_invitation_rejected_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named(u'J\xf6hn', u'Doe'))
        user = create(Builder('user').named(u'Hugo', u'B\xf6ss'))
        Invitation(target=folder,
                   email=user.getProperty('email'),
                   inviter=inviter.getId(),
                   roles=['Reader'])
        transaction.commit()

        browser.login(user).open(view='invitations')
        browser.click_on("No, I reject the invitation")

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:invitation_rejected',
                 'actor': 'hugo.boss',
                 'path': '/plone/folder',
                 'invitation:inviter': 'john.doe',
                 'invitation:email': 'hugo@boss.com',
                 'invitation:roles': ('Reader',),
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'invitation:inviter',
                                 'invitation:email',
                                 'invitation:roles',)))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Invitation rejected now by B\xf6ss Hugo',
             'title': ''},
            newest_event.infos())

    @browsing
    def test_invitation_retracted_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        inviter = create(Builder('user').named(u'J\xf6hn', u'Doe'))
        invitation = Invitation(target=folder,
                                email='foo@bar.com',
                                inviter=inviter.getId(),
                                roles=['Reader'])
        transaction.commit()

        browser.login(inviter).open(view='invitations')
        browser.fill({'sent_invitations:list': invitation.iid})
        browser.click_on('Retract')

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:invitation_retracted',
                 'actor': 'john.doe',
                 'path': '/plone/folder',
                 'invitation:inviter': 'john.doe',
                 'invitation:email': 'foo@bar.com',
                 'invitation:roles': ('Reader',),
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'invitation:inviter',
                                 'invitation:email',
                                 'invitation:roles',)))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Invitation retracted now by Doe J\xf6hn',
             'title': ''},
            newest_event.infos())
        self.assertEquals(
            u'Doe J\xf6hn has retracted the invitation for foo@bar.com.',
            newest_event.body_text)

    @browsing
    def test_roles_changed_event(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))

        hugo = create(Builder('user').named(u'Hugo', u'B\xf6ss')
               .with_roles('Manager', on=folder))
        create(Builder('user').named(u'J\xf6hn', u'Doe')
               .with_roles('Editor', on=folder))

        browser.login(hugo).open(folder, view='participants')
        browser.click_on('change')
        # Only Editor and Contributor are selectable
        browser.fill({'Roles': ['Contributor']}).submit()

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:role_changed',
                 'actor': 'hugo.boss',
                 'path': '/plone/folder',
                 'roles:userid': 'john.doe',
                 'roles:old': ('Editor',),
                 'roles:removed': ('Editor',),
                 'roles:new': ('Contributor',),
                 'roles:added': ('Contributor',),
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'roles:userid',
                                 'roles:old',
                                 'roles:removed',
                                 'roles:new',
                                 'roles:added')))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Role changed now by B\xf6ss Hugo',
             'title': ''},
            newest_event.infos())
        self.assertEquals(
            u'B\xf6ss Hugo has changed the role of Doe J\xf6hn'
            u' from Can edit to Can add.',
            newest_event.body_text)

    @browsing
    def test_local_role_removed(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').providing(IParticipationSupport))
        hugo = create(Builder('user').named(u'Hugo', u'B\xf6ss')
               .with_roles('Manager', on=folder))
        user = create(Builder('user').named(u'J\xf6hn', u'Doe')
                      .with_roles('Editor', on=folder))

        browser.login(hugo).open(folder, view='participants')
        browser.fill({'userids:list': [user.getId()]}) \
               .find('Delete Participants').click()

        self.maxDiff = None
        self.assertEquals(
            [
                {'action': 'added',
                 'actor': 'test_user_1_',
                 'path': '/plone/folder'},

                {'action': 'participation:role_removed',
                 'actor': 'hugo.boss',
                 'path': '/plone/folder',
                 'roles:userid': 'john.doe',
                },
            ],

            get_soup_activities(('path',
                                 'action',
                                 'actor',
                                 'roles:userid')))

        browser.open(folder, view='@@activity')
        newest_event = activity.events()[0]
        self.assertEquals(
            {'url': 'http://nohost/plone/folder',
             'byline': u'Participant removed now by B\xf6ss Hugo',
             'title': ''},
            newest_event.infos())
        self.assertEquals(
            u'B\xf6ss Hugo has removed the participant Doe J\xf6hn.',
            newest_event.body_text)



from AccessControl import SpecialUsers
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from ftw.participation import _
from ftw.participation.interfaces import IInvitationStorage
from zExceptions import Forbidden
import os.path


class InvitationsView(BrowserView):
    """The invitations shows a listing of invitations the user got (where he
    was invited by another user) and a listing of invitations where he invited
    someone else.

    """

    template = ViewPageTemplateFile('invitations.pt')

    def __call__(self):
        self.storage = IInvitationStorage(self.context)
        self.handle_pending_invitations()
        authenticator = self.context.restrictedTraverse('@@authenticator',
                                                        None)
        if not self.is_logged_in():
            # redirect to home when no
            pending_iids = self.storage.get_pending_session_invitations()
            if not len(pending_iids):
                # no pending sessions and not logged in -> go to home
                url = self.context.portal_url()
                return self.request.RESPONSE.redirect(url)

            else:
                # check if the email of at least one pending invitation is
                # already used by any user -> then the current user may be
                # this user, so we redirect to the login form
                passearch = self.context.unrestrictedTraverse('@@pas_search')
                for iid in pending_iids:
                    invitation = self.storage.get_invitation_by_iid(iid)
                    if not invitation:
                        # Maybe a deleted Invitation is still in the session.
                        continue
                    if len(passearch.searchUsers(email=invitation.email)) > 0:
                        url = os.path.join(
                            self.context.portal_url(),
                            'login_form?came_from=@@invitations')
                        return self.request.RESPONSE.redirect(url)
                # if we cannot find it we redirect to the invite_join_form
                url = os.path.join(self.context.portal_url(),
                                   'welcome_invited')
                # store one of the email addresses to the session. it can be
                # used later while registering as proposal
                if invitation:
                    self.request.SESSION['invite_email'] = invitation.email
                return self.request.RESPONSE.redirect(url)

        # not logged in -> show the view...
        else:
            # if we have pending session invitations, we should assign them to
            # the current user now.
            self.storage.assign_pending_session_invitations()
            if self.request.get('form.submitted', False):
                # sanity check with authenticator
                if not authenticator.verify():
                    raise Forbidden

                # handle accepted invitations
                if self.request.get('accept', False):
                    for iid in self.request.get('received_invitations', []):
                        self.context.unrestrictedTraverse(
                            '@@accept_invitation')(iid)

                # handle rejected invitations
                if self.request.get('reject', False):
                    for iid in self.request.get('received_invitations', []):
                        self.context.unrestrictedTraverse(
                            '@@reject_invitation')(iid)

                # hande reetracted invitations
                if self.request.get('retract', False):
                    for iid in self.request.get('sent_invitations', []):
                        self.context.unrestrictedTraverse(
                            '@@retract_invitation')(iid)

        return self.template()

    def get_received_invitations(self):
        """Returns all invitations which the user has received and open.
        """
        return self.storage.get_invitations_for_email()

    def get_sent_invitations(self):
        """Returns all open invitations created by the current user.
        """
        return self.storage.get_invitations_invited_by()

    def handle_pending_invitations(self):
        """When the `iid` of a invitation is in the request
        (@@invitaitons?iid=bla) and the user is not authenticated, we mark this
        invitation in the session. If there is a `iid` but the user is
        authenticated we can reassign the invitation to this user.

        """
        iid = self.request.get('iid', False)
        if not iid:
            return
        invitation = self.storage.get_invitation_by_iid(iid)
        if not invitation:
            # the invitation is expired or answered
            msg = _(u'warning_invitaiton_expired',
                    default=u'The invitation has expired.')
            IStatusMessage(self.request).addStatusMessage(msg, type='warning')
            return

        if self.is_logged_in():
            # reassign to current user
            mtool = getToolByName(self.context, 'portal_membership')
            member = mtool.getAuthenticatedMember()
            new_email = member.getProperty('email', False) or member.getId()
            self.storage.reassign_invitation(invitation, new_email)
        else:
            # set pending for session
            self.storage.set_invitation_pending_for_session(invitation)

    def is_logged_in(self):
        """Returns wheather the user is logged in or not.
        """
        mtool = getToolByName(self.context, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        return member != SpecialUsers.nobody



from ftw.participation import _
from ftw.participation.browser.accept import AcceptInvitation
from ftw.participation.events import InvitationRejectedEvent
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from zope.event import notify
from zope.i18n import translate
import os.path


class RejectInvitation(AcceptInvitation):
    """Reject a invitation. Like accept but with different messages
    and without partipating.

    """

    def __call__(self, iid=None):
        """
        """
        if not iid:
            iid = self.request.get('iid')

        self.load(iid)
        notify(InvitationRejectedEvent(self.target, self.invitation))

        self.send_email()

        # add status message
        msg = _(u'info_invitation_rejected',
                default=u'You have rejected the invitation.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')

        # destroy the invitation
        self.storage.remove_invitation(self.invitation)
        del self.invitation

        # redirect back
        url = self.request.get('HTTP_REFERER',
                               os.path.join(self.context.portal_url(),
                                            '@@invitations'))
        return self.request.RESPONSE.redirect(url)

    def get_mail_body_html_view(self):
        return self.context.unrestrictedTraverse(
            '@@invitation_rejected_mail_html')

    def get_mail_body_text_view(self):
        return self.context.unrestrictedTraverse(
            '@@invitation_rejected_mail_text')

    def get_subject(self):
        """Returns the translated subject of the email.
        """
        member = getToolByName(self.context,
                               'portal_membership').getAuthenticatedMember()
        fullname = member.getProperty(
            'fullname', member.getId()).decode('utf8')
        context_title = self.context.pretty_title_or_id().decode('utf-8')
        # -- i18ndude hint --
        if 0:
            _(u'mail_invitation_rejected_subject',
              default=u'The Invitation to participate in ${title} '
              u'was rejected by ${user}',
              mapping=dict(title=context_title,
                           user=fullname))
        # / -- i18ndude hint --
        # do not use translation messages - translate directly
        return translate(u'mail_invitation_rejected_subject',
                         domain='ftw.participation',
                         context=self.request,
                         default=u'The Invitation to participate in ${title} '
                         u'was rejected by ${user}',
                         mapping=dict(title=context_title,
                                      user=fullname))



from plone.principalsource import source
from plone.principalsource.term import PrincipalTerm


class ParticipationUsers(source.PrincipalSource):

    def _term_for_result(self, result_dict):
        id = result_dict['id']

        token = id
        if isinstance(token, unicode):
            token = token.encode('utf-8')

        type = result_dict.get('principal_type', 'user')
        value = result_dict.get('login', result_dict.get('groupid')) or id
        title = result_dict.get('title') or value

        # Attempt to get a title from the fullname if not set. Unfortunately,
        # source_users doesn't have fullname, and mutable_properties doesn't
        # match on login name or id when searching.

        if title == value:
            if type == 'user':
                user = self.acl_users.getUserById(id)
                if user is not None:
                    try:
                        # XXX: user.getProperty() is PlonePAS specfic
                        title = user.getProperty('fullname') or value
                        if user.getProperty('email'):
                            title += ' (%s)' % user.getProperty('email')
                    except AttributeError:
                        pass
        return PrincipalTerm(value=value, type=type, token=token, title=title)


class ParticipationBinder(source.PrincipalSourceBinder):

    def __call__(self, context):
        return ParticipationUsers(context, self.users, self.groups)


ParticipationUsersFactory = ParticipationBinder(users=True, groups=False)



from ftw.activity.catalog import get_activity_soup
from ftw.activity.catalog.record import ActivityRecord


def create_record_from_invitation(obj, invitation, action,
                                  actor_userid=None, date=None):
    record = ActivityRecord(obj, action,
                            actor_userid=actor_userid, date=date)
    record.attrs['invitation:inviter'] = invitation.inviter
    record.attrs['invitation:email'] = invitation.email
    record.attrs['invitation:roles'] = tuple(invitation.roles)
    return record


def invitation_created(obj, event, actor_userid=None, date=None):
    record = create_record_from_invitation(
        obj, event.invitation, 'participation:invitation_created')
    record.attrs['invitation:comment'] = event.comment
    return get_activity_soup().add(record)


def invitation_accepted(obj, event, actor_userid=None, date=None):
    record = create_record_from_invitation(
        obj, event.invitation, 'participation:invitation_accepted')
    return get_activity_soup().add(record)


def invitation_rejected(obj, event, actor_userid=None, date=None):
    record = create_record_from_invitation(
        obj, event.invitation, 'participation:invitation_rejected')
    return get_activity_soup().add(record)


def invitation_retracted(obj, event, actor_userid=None, date=None):
    record = create_record_from_invitation(
        obj, event.invitation, 'participation:invitation_retracted')
    return get_activity_soup().add(record)


def role_changed(obj, event, actor_userid=None, date=None):
    record = ActivityRecord(obj, 'participation:role_changed',
                            actor_userid=actor_userid, date=date)
    record.attrs['roles:userid'] = event.userid
    record.attrs['roles:old'] = tuple(event.old_roles)
    record.attrs['roles:new'] = tuple(event.new_roles)
    record.attrs['roles:removed'] = tuple(
        set(event.old_roles) - set(event.new_roles))
    record.attrs['roles:added'] = tuple(
        set(event.new_roles) - set(event.old_roles))
    return get_activity_soup().add(record)


def role_removed(obj, event, actor_userid=None, date=None):
    record = ActivityRecord(obj, 'participation:role_removed',
                            actor_userid=actor_userid, date=date)
    record.attrs['roles:userid'] = event.userid
    return get_activity_soup().add(record)



from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from ftw.participation import _
from ftw.participation.events import InvitationAcceptedEvent
from ftw.participation.interfaces import IInvitationStorage
from ftw.participation.interfaces import IParticipationSetter
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import Redirect
from zope.component import getMultiAdapter, getUtility
from zope.event import notify
from zope.i18n import translate


class AcceptInvitation(BrowserView):
    """Accept a invitation. This either called directly from the browser (with
    a url like .../@@accept_invitations?iid=XXX) or through the invitations
    view (with unrestrictedTravers('@@accept_invitations')(iid) ).

    """

    def __call__(self, iid=None, redirect=True):
        if not iid:
            iid = self.request.get('iid')

        if not self.load(iid):
            msg = _(u'error_invitation_not_found',
                    default=u'Could not find the invitation.')
            IStatusMessage(self.request).addStatusMessage(msg, type='error')
            raise Redirect(self.context.portal_url())

        self.set_participation()
        notify(InvitationAcceptedEvent(self.target, self.invitation))
        self.send_email()

        # add status message
        msg = _(u'info_invitation_accepted',
                default=u'You have accepted the invitation.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')

        # destroy the invitation
        self.storage.remove_invitation(self.invitation)
        del self.invitation

        # redirect context (where the user now has access)
        if redirect:
            return self.request.RESPONSE.redirect(self.target.absolute_url())

    def load(self, iid):
        """Loads the storage, the invitation and the target and does some
        more things.

        """
        self.storage = IInvitationStorage(self.context)
        self.invitation = self.storage.get_invitation_by_iid(iid)
        if not self.invitation:
            return False

        self.target = self.invitation.get_target()

        # sanity check
        mtool = getToolByName(self.context, 'portal_membership')
        self.member = mtool.getAuthenticatedMember()
        valid_emails = (self.member.getId(),
                        self.member.getProperty('email', object()))
        if self.invitation.email not in valid_emails:
            return False

        return True

    def set_participation(self):
        """Sets up participation up.
        """
        # participate the user (using the adapter)
        setter = getMultiAdapter((self.target, self.request,
                                  self.invitation, self.member),
                                 IParticipationSetter)
        setter()

    def send_email(self):
        """Sends the notification email to the invitor.
        """
        properties = getUtility(IPropertiesTool)
        mtool = getToolByName(self.context, 'portal_membership')
        # prepare from address for header
        from_str = Header(properties.email_from_name, 'utf-8')
        from_str.append(u'<%s>' % properties.email_from_address.decode('utf-8'))

        # To
        to_member = mtool.getMemberById(self.invitation.inviter)
        if not to_member.getProperty('email'):
            msg = _(u'error_inviter_email_missing',
                    default=u'Invitor could not be notified: he has '
                    'no email address defined.')
            IStatusMessage(self.request).addStatusMessage(msg, type='error')
            return
        to_str = to_member.getProperty('fullname', to_member.getId())
        if isinstance(to_str, unicode):
            to_str = to_str.encode('utf8')
        header_to = Header(to_str, 'utf-8')
        header_to.append(u'<%s>' % to_member.getProperty(
                'email').decode('utf-8'))
        # Subject
        subject = self.get_subject()
        if isinstance(subject, unicode):
            subject = subject.encode('utf8')
        header_subject = Header(subject, 'utf8')

        # Options for templated
        options = {
            'invitation': self.invitation,
            'inviter': to_member,
            'inviter_name': to_member.getProperty('fullname',
                                                  to_member.getId()),
            'invited': self.member,
            'invited_name': self.member.getProperty('fullname',
                                                    self.member.getId()),
            'invited_email': self.member.getProperty('email',
                                                     self.member.getId()),
            'target': self.target,
            }

        # make the mail
        msg = MIMEMultipart('alternative')
        msg['Subject'] = header_subject
        msg['From'] = str(from_str)
        msg['To'] = str(header_to)
        # get the body views
        html_view = self.get_mail_body_html_view()
        text_view = self.get_mail_body_text_view()

        # render and emmbed body
        text_body = text_view(**options).encode('utf-8')
        msg.attach(MIMEText(text_body, 'plain', 'utf-8'))
        html_body = html_view(**options).encode('utf-8')
        msg.attach(MIMEText(html_body, 'html', 'utf-8'))

        # send the email
        mh = getToolByName(self.context, 'MailHost')
        mh.send(msg)

    def get_mail_body_html_view(self):
        return self.context.unrestrictedTraverse(
            '@@invitation_accepted_mail_html')

    def get_mail_body_text_view(self):
        return self.context.unrestrictedTraverse(
            '@@invitation_accepted_mail_text')

    def get_subject(self):
        """Returns the translated subject of the email.
        """
        member = getToolByName(self.context,
                               'portal_membership').getAuthenticatedMember()
        fullname = member.getProperty('fullname',
                                      member.getId()).decode('utf-8')
        context_title = self.context.pretty_title_or_id().decode('utf-8')
        # -- i18ndude hint --
        if 0:
            _(u'mail_invitation_accepted_subject',
              default=u'The Invitation to participate in ${title} '
              u'was accepted by ${user}',
              mapping=dict(title=context_title,
                           user=fullname))
        # / -- i18ndude hint --
        # do not use translation messages - translate directly
        return translate(u'mail_invitation_accepted_subject',
                         domain='ftw.participation',
                         context=self.request,
                         default=u'The Invitation to participate in ${title} '
                         u'was accepted by ${user}',
                         mapping=dict(title=context_title,
                                      user=fullname))



from setuptools import setup, find_packages
import os

version = '1.5.4.dev0'
maintainer = 'Jonas Baumann'

tests_require = (
    'zope.testing',
    'plone.app.testing',
    'zope.globalrequest', # Because of TinyMCE
    'ftw.activity > 2',
    'ftw.builder',
    'ftw.testbrowser',
    'ftw.testing',
    'ftw.builder',
    'ftw.tabbedview',
    'Products.PloneHotfix20160830',
    )


extras_require = {
    'tests': tests_require,
    'activity': [
        'ftw.activity > 2',
    ]
}


setup(name='ftw.participation',
      version=version,
      description='Invite other users (registered or unregistered) to a ' + \
          'context in plone.',
      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),


      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw participation plone',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.participation',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'Plone',
        'archetypes.schemaextender',
        'ftw.upgrade>=1.6.0',
        'plone.api',
        'plone.app.registry',
        'plone.formwidget.autocomplete',
        'plone.principalsource',
        'plone.z3cform',
        'setuptools',
        'z3c.form',
        ],

      tests_require=tests_require,
      extras_require=extras_require,
      entry_points='''
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      ''',
      )



from ftw.activity.browser.renderer import DefaultRenderer
from zope.component.hooks import getSite
from ftw.participation import _
from ftw.participation.activity.utils import translate_and_join_roles
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class InvitationRenderer(DefaultRenderer):
    index = ViewPageTemplateFile('templates/invitation.pt')

    def position(self):
        return 900

    def match(self, activity, obj):
        return activity.attrs['action'].startswith('participation:')

    def render(self, activity, obj):
        return self.index(activity=activity,
                          obj=obj,
                          comment=self.prepare_comment(activity, obj))

    def prepare_comment(self, activity, obj):
        action = activity.attrs['action']
        actor_info = activity.get_actor_info()
        actor_fullname = actor_info.get('fullname').decode('utf-8')

        if action == 'participation:invitation_created':
            actor_email = activity.attrs['invitation:email'].decode('utf-8')
            roles = translate_and_join_roles(
                activity.attrs['invitation:roles'], obj, self.request)

            return _(u'activity_invitation_created_body',
                     default=u'${inviter} has invited ${mail} as ${roles}.',
                     mapping={'inviter': actor_fullname,
                              'mail': actor_email,
                              'roles': roles})

        if action == 'participation:invitation_retracted':
            actor_email = activity.attrs['invitation:email'].decode('utf-8')
            roles = translate_and_join_roles(
                activity.attrs['invitation:roles'], obj, self.request)

            return _(u'activity_invitation_retracted_body',
                     default=u'${actor} has retracted the invitation'
                     u' for ${mail}.',
                     mapping={'actor': actor_fullname,
                              'mail': actor_email})

        if action == 'participation:role_changed':
            subject_fullname = self.get_fullname_of(
                activity.attrs['roles:userid'])
            removed_roles = translate_and_join_roles(
                activity.attrs['roles:removed'], obj, self.request)
            added_roles = translate_and_join_roles(
                activity.attrs['roles:added'], obj, self.request)

            return _(u'activity_role_changed_body',
                     default=u'${actor} has changed the role of ${subject}'
                     u' from ${removed_roles} to ${added_roles}.',
                     mapping={'actor': actor_fullname,
                              'subject': subject_fullname,
                              'removed_roles': removed_roles,
                              'added_roles': added_roles})

        if action == 'participation:role_removed':
            subject_fullname = self.get_fullname_of(
                activity.attrs['roles:userid'])

            return _(u'activity_participant_removed_body',
                     default=u'${actor} has removed the'
                     u' participant ${subject}.',
                     mapping={'actor': actor_fullname,
                              'subject': subject_fullname})

        return None

    def get_fullname_of(self, userid):
        membership = getToolByName(getSite(), 'portal_membership')
        member = membership.getMemberById(userid)
        if not member:
            result = userid or 'N/A'
        else:
            result = member.getProperty('fullname') or userid

        if not isinstance(result, unicode):
            result = result.decode('utf-8')

        return result



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationRegistry
from ftw.participation.interfaces import IParticipationSupport
from ftw.participation.tests.layer import FTW_PARTICIPATION_FUNCTIONAL_TESTING
from ftw.participation.tests.pages import participants_view
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from ftw.testing.mailing import Mailing
from plone.app.testing import login
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.component import getUtility
import transaction


class TestParticipantsView(TestCase):

    layer = FTW_PARTICIPATION_FUNCTIONAL_TESTING

    def setUp(self):
        Mailing(self.layer['portal']).set_up()
        self.john = create(Builder('user')
                           .named('John', 'Doe')
                           .with_roles('Contributor'))

        login(self.layer['portal'], self.john.getId())
        self.folder = create(Builder('folder')
                             .titled('The Folder')
                             .providing(IParticipationSupport))

    def tearDown(self):
        Mailing(self.layer['portal']).tear_down()

    @browsing
    def test_participants_are_listed_in_table(self, browser):
        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Reader', 'Contributor', on=self.folder))

        browser.login().visit(self.folder, view='participants')
        self.assertIn({u'': '',
                       u'User': u'Boss Hugo (hugo@boss.com)',
                       u'Roles': u'Can add, Can view',
                       u'Invited by': u'',
                       u'Status': u'Accepted',
                       u'': u'change'},
                      participants_view.table())
    @browsing
    def test_participants_with_inherited_roles_are_listed_in_table(self, browser):
        subfolder = create(Builder('folder')
                           .titled('The SubFolder')
                           .within(self.folder)
                           .providing(IParticipationSupport))

        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Reader', 'Contributor', on=self.folder))

        browser.login().visit(subfolder, view='participants')

        self.assertEquals(2,
                          len(browser.css('.roleInherited')),
                          'Expect two inherited roles')

        self.assertIn({u'': '',
                       u'User': u'Boss Hugo (hugo@boss.com)',
                       u'Roles': u'Can add, Can view',
                       u'Invited by': u'',
                       u'Status': u'Accepted',
                       u'': u'change'},
                      participants_view.table())

    @browsing
    def test_invitations_are_listed_in_table(self, browser):
        hugo = create(Builder('user').named('Hugo', 'Boss'))
        fraenzi = create(Builder('user')
                         .named('Fr\xc3\xa4nzi', 'M\xc3\xbcller'))

        create(Builder('invitation')
               .inviting(fraenzi)
               .to(self.folder)
               .invited_by(hugo))

        browser.login().visit(self.folder, view='participants')
        self.assertIn({u'': '',
                       'User': 'franzi@muller.com',
                       'Roles': 'Can view',
                       'Invited by': 'Boss Hugo',
                       'Status': 'Pending',
                       u'': u''},
                      participants_view.table())

    @browsing
    def test_user_sorting_works_with_umlauts_in_names(self, browser):
        fraenzi = create(Builder('user')
                         .named('Fr\xc3\xa4nzi', 'M\xc3\xbcller')
                         .with_roles('Reader', on=self.folder))

        juergen = create(Builder('user')
                         .named('J\xc3\xbcrgen', 'R\xc3\xbcegsegger'))

        create(Builder('invitation')
               .inviting(juergen)
               .to(self.folder)
               .invited_by(fraenzi))

        browser.login().visit(self.folder, view='participants')
        self.assertEquals([u'Doe John (john@doe.com)',
                           u'jurgen@ruegsegger.com',
                           u'M\xfcLler Fr\xe4Nzi (franzi@muller.com)'],
                          participants_view.users_column())

    @browsing
    def test_user_with_umlaut_and_no_email_is_working(self, browser):
        fraenzi = create(Builder('user')
                         .named('Fr\xc3\xa4nzi', 'M\xc3\xbcller')
                         .with_roles('Reader', on=self.folder))
        fraenzi.setMemberProperties({'email': ''})
        transaction.commit()

        browser.login().visit(self.folder, view='participants')
        self.assertEquals([u'Doe John (john@doe.com)',
                           u'M\xfcLler Fr\xe4Nzi'],
                          participants_view.users_column())

    @browsing
    def test_owner_can_remove_participants(self, browser):
        hugo = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_roles('Reader', on=self.folder))
        hugos_name = u'Boss Hugo (hugo@boss.com)'

        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        self.assertIn(hugos_name, participants_view.users_column())

        browser.fill({'userids:list': [hugo.getId()]})
        browser.find('Delete Participants').click()
        self.assertNotIn(hugos_name, participants_view.users_column())

    @browsing
    def test_owner_can_invite_new_users(self, browser):
        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        browser.find('Invite participants').click()
        self.assertEquals('invite_participants', plone.view())

    @browsing
    def test_cancel_redirects_to_default_view(self, browser):
        browser.login().visit(self.folder, view='participants')
        browser.find('Cancel').click()
        self.assertRegexpMatches(browser.url, r'/the-folder$')

    @browsing
    def test_checkboxes_only_visible_for_privileged_users(self, browser):
        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        self.assertTrue(participants_view.checkboxes_visible())

        hugo = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_roles('Reader', on=self.folder))
        browser.login(hugo.getId()).visit(self.folder, view='participants')
        self.assertFalse(participants_view.checkboxes_visible())

    @browsing
    def test_delete_button_only_visible_for_privileged_users(self, browser):
        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        self.assertTrue(browser.find('Delete Participants'))

        hugo = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_roles('Reader', on=self.folder))
        browser.login(hugo.getId()).visit(self.folder, view='participants')
        self.assertFalse(browser.find('Delete Participants'))

    @browsing
    def test_invite_link_only_visible_for_privileged_users(self, browser):
        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        self.assertTrue(browser.find('Invite participants'))

        hugo = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_roles('Reader', on=self.folder))
        browser.login(hugo.getId()).visit(self.folder, view='participants')
        self.assertFalse(browser.find('Invite participants'))

    @browsing
    def test_unprivileged_users_cannot_delete_participants(self, browser):
        # This is a security test.
        # Since normal users do not see the UI for deletion, we use a
        # privileged user for renering the UI and switch to a non-privileged
        # user for sending the delete request.

        jane = create(Builder('user')
                      .named('Jane', 'Doe')
                      .with_roles('Reader', on=self.folder))

        hugo = create(Builder('user')
                      .named('Hugo', 'Boss')
                      .with_roles('Reader', on=self.folder))

        browser.login(self.john.getId()).visit(self.folder,
                                               view='participants')
        browser.fill({'userids:list': [jane.getId()]})
        browser.login(hugo.getId())

        with browser.expect_http_error(reason='Forbidden'):
            browser.find('Delete Participants').click()

    @browsing
    def test_participantstab_works_without_participationsupport(self, browser):
        folder2 = create(Builder('folder')
                         .titled('The Folder'))

        browser.login().visit(folder2, view='tabbedview_view-participants')

        self.assertFalse(browser.css('[href*="@@invite_participants"]'),
                         'Invite link should not be visible')
        self.assertFalse(browser.css('[name="form.delete"]'),
                         'Delete button should not be visible')
        self.assertFalse(browser.css('[name="form.cancel"]'),
                         'Cancel button should not be visible')

        self.assertNotIn('Invited by',
                         browser.css('table').first.head_rows.text,
                         '"Invited by" column should no be visible')
        self.assertNotIn('State',
                         browser.css('table').first.head_rows.text,
                         '"State" column should no be visible')

        self.assertNotIn('change',
                         browser.css('table').first.lists()[1],
                         'The "change" role link should not be visible')
        self.assertFalse(browser.css('table input[name="checkbox"]'),
                         'There should be no column with checkboxes')

    @browsing
    def test_participationtab_works_if_ext_and_int_is_disabled(self, browser):
        registry = getUtility(IRegistry)
        config = registry.forInterface(IParticipationRegistry)
        config.allow_invite_users = False
        config.allow_invite_email = False
        transaction.commit()

        browser.login().visit(self.folder, view='tabbedview_view-participants')

        self.assertFalse(browser.css('[href*="@@invite_participants"]'),
                         'Invite link should not be visible')



from ftw.participation import _
from ftw.participation.browser.accept import AcceptInvitation
from ftw.participation.events import InvitationRetractedEvent
from ftw.participation.interfaces import IInvitationStorage
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import Redirect
from zope.event import notify
import os.path


class RetractInvitation(AcceptInvitation):
    """The inviter retracts the invitation.
    """

    def __call__(self, iid=None):
        if not iid:
            iid = self.request.get('iid')

        self.load(iid)
        notify(InvitationRetractedEvent(self.target, self.invitation))

        msg = _(u'info_invitation_retracted',
                default=u'You have retracted the invitation.')
        IStatusMessage(self.request).addStatusMessage(msg)

        # destroy the invitation
        self.storage.remove_invitation(self.invitation)
        del self.invitation

        # redirect back
        url = self.request.get('HTTP_REFERER',
                               os.path.join(self.context.portal_url(),
                                            '@@invitations'))
        return self.request.RESPONSE.redirect(url)

    def load(self, iid):
        """Loads storage, invitation, target, etc
        """
        self.storage = IInvitationStorage(self.context)
        self.invitation = self.storage.get_invitation_by_iid(iid)
        self.target = self.invitation.get_target()

        # sanity check
        mtool = getToolByName(self.context, 'portal_membership')
        self.member = mtool.getAuthenticatedMember()
        if not self.invitation \
                or self.invitation.inviter != self.member.getId():
            msg = _(u'error_invitation_not_found',
                    default=u'Could not find the invitation.')
            IStatusMessage(self.request).addStatusMessage(msg, type='error')
            raise Redirect(self.context.portal_url())



from ftw.participation.browser.participants import ManageParticipants


class ParticipantsTab(ManageParticipants):

    def __call__(self):

        return self.form()

    def hide_cancel_button(self):
        return True



from ftw.participation.interfaces import IInvitationAcceptedEvent
from ftw.participation.interfaces import IInvitationCreatedEvent
from ftw.participation.interfaces import IInvitationEvent
from ftw.participation.interfaces import IInvitationRejectedEvent
from ftw.participation.interfaces import IInvitationRetractedEvent
from ftw.participation.interfaces import ILocalRoleRemoved
from ftw.participation.interfaces import IRolesChangedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class InvitationEvent(ObjectEvent):
    """An event related to an invitation.
    """
    implements(IInvitationEvent)

    def __init__(self, object, invitation):
        self.object = object
        self.invitation = invitation


class InvitationCreatedEvent(InvitationEvent):
    """An invitation was created.
    """
    implements(IInvitationCreatedEvent)

    def __init__(self, object, invitation, comment):
        self.object = object
        self.invitation = invitation
        self.comment = comment


class InvitationAcceptedEvent(InvitationEvent):
    """An invitation was accepted.
    """
    implements(IInvitationAcceptedEvent)


class InvitationRejectedEvent(InvitationEvent):
    """An invitation was rejected.
    """
    implements(IInvitationRejectedEvent)


class InvitationRetractedEvent(InvitationEvent):
    """An invitation was retracted.
    """
    implements(IInvitationRetractedEvent)


class RolesChangedEvent(object):
    """The roles of a participant was changed.
    """
    implements(IRolesChangedEvent)

    def __init__(self, object, userid, old_roles, new_roles):
        self.object = object
        self.userid = userid
        self.old_roles = old_roles
        self.new_roles = new_roles


class LocalRoleRemoved(object):
    implements(ILocalRoleRemoved)

    def __init__(self, object, userid):
        self.object = object
        self.userid = userid
